/****************************************************************************
** Dooble - The Secure Internet Web Browser
**
** Copyright (c) 2008, 2009, 2010, 2011 Alexis Megas,
** Gunther van Dooble, and the Dooble Team.
** All rights reserved.
**
** License: GPL2 only:
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; version 2 of the License only.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
** or see here: http://www.gnu.org/licenses/gpl.html
**
** For the WebKit library, please see: http://webkit.org.
**
** THE CODE IS PROVIDED BY THE AUTHORS ''AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
** ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
** IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS APPLICATION, EVEN IF ADVISED
** OF THE POSSIBILITY OF SUCH DAMAGE.
**
** Please report all praise, requests, bugs, and problems to the project
** team and administrators: http://sf.net/projects/dooble.
**
** You can find us listed at our project page. New team members are welcome.
** The name of the authors should not be used to endorse or promote products
** derived from Dooble without specific prior written permission.
** If you use this code for other projects, please let us know.
**
** Web sites:
**   http://sf.net/projects/dooble
**   http://dooble.sf.net
****************************************************************************/

#include <QDir>
#include <QMap>
#include <QUrl>
#include <QLocale>
#include <QKeyEvent>
#include <QSettings>
#include <QTextCodec>
#include <QFileDialog>
#include <QMessageBox>
#include <QWebSettings>
#include <QProgressDialog>
#include <QDesktopServices>

#include "dmisc.h"
#include "dooble.h"
#include "dsettings.h"

dsettings::dsettings():QMainWindow()
{
  m_parentDooble = 0;
  ui.setupUi(this);
  ui.rememberClosedTabsSpinBox->setMaximum
    (dooble::MAX_HISTORY_ITEMS);
#ifdef Q_WS_MAC
  setAttribute(Qt::WA_MacMetalStyle, true);

  /*
  ** Move the title so that the box's border remains clean.
  */

  foreach(QGroupBox *groupBox, findChildren<QGroupBox *> ())
    groupBox->setStyleSheet("QGroupBox::title {"
			    "top: -1px;"
			    "subcontrol-origin: border;}");
#endif

  foreach(QPushButton *button, ui.buttonBox->findChildren<QPushButton *>())
    button->setIcon(QIcon());

  QSettings settings;

  if(!dooble::s_settings.contains("iconSet"))
    {
      settings.setValue("iconSet",
			QString("%1/%2").arg(QDir::currentPath()).
			arg("Icons/nuovext/configuration.cfg"));
      dooble::s_settings["iconSet"] =
	QString("%1/%2").arg(QDir::currentPath()).
	arg("Icons/nuovext/configuration.cfg");
    }

  if(!dooble::s_settings.contains("settingsWindow/iconSet1"))
    {
      settings.setValue("settingsWindow/iconSet1", settings.value("iconSet"));
      dooble::s_settings["settingsWindow/iconSet1"] =
	dooble::s_settings["iconSet"];
    }

  QSettings cfgSettings(dooble::s_settings.value("iconSet").toString(),
			QSettings::IniFormat);

  setWindowIcon
    (QIcon(cfgSettings.value("settingsWindow/windowIcon").toString()));
  connect(ui.buttonBox, SIGNAL(clicked(QAbstractButton *)),
	  this, SLOT(slotClicked(QAbstractButton *)));
  connect(ui.chooseIconSetPushButton, SIGNAL(clicked(void)),
	  this, SLOT(slotSelectIconCfgFile(void)));
  connect(ui.chooseMyRetrievedFilesPushButton, SIGNAL(clicked(void)),
	  this, SLOT(slotChooseMyRetrievedFilesDirectory(void)));
  connect(ui.fontComboBox, SIGNAL(currentIndexChanged(const QString &)),
	  this, SLOT(slotFontChanged(const QString &)));
  connect(ui.defaultFontCombinationBox,
	  SIGNAL(currentIndexChanged(const QString &)),
	  this, SLOT(slotWebFontChanged(const QString &)));
  connect(ui.fixedFontCombinationBox,
	  SIGNAL(currentIndexChanged(const QString &)),
	  this, SLOT(slotWebFontChanged(const QString &)));
  connect(ui.historyCheckBox,
	  SIGNAL(clicked(bool)),
	  ui.historySpinBox,
	  SLOT(setEnabled(bool)));
  connect(ui.cookieTimerComboBox,
	  SIGNAL(currentIndexChanged(int)),
	  this,
	  SLOT(slotCookieTimerTimeChanged(int)));
  connect(ui.cookieTimerCheckBox,
	  SIGNAL(clicked(bool)),
	  ui.cookieTimerComboBox,
	  SLOT(setEnabled(bool)));
  connect(ui.cookieTimerCheckBox,
	  SIGNAL(clicked(bool)),
	  ui.cookieTimerSpinBox,
	  SLOT(setEnabled(bool)));
  connect(ui.changePassphrasePushButton,
	  SIGNAL(clicked(bool)),
	  this,
	  SLOT(slotEnablePassphrase(void)));
  connect(ui.browsingProxyManualRadio,
	  SIGNAL(toggled(bool)),
	  ui.browsingFtpProxyGroupBox,
	  SLOT(setEnabled(bool)));
  connect(ui.browsingProxyManualRadio,
	  SIGNAL(toggled(bool)),
	  ui.browsingHttpProxyGroupBox,
	  SLOT(setEnabled(bool)));
  connect(ui.downloadProxyManualRadio,
	  SIGNAL(toggled(bool)),
	  ui.downloadFtpProxyGroupBox,
	  SLOT(setEnabled(bool)));
  connect(ui.downloadProxyManualRadio,
	  SIGNAL(toggled(bool)),
	  ui.downloadHttpProxyGroupBox,
	  SLOT(setEnabled(bool)));
  connect(ui.cookieGroupBox,
	  SIGNAL(clicked(bool)),
	  this,
	  SLOT(slotGroupBoxClicked(bool)));
  connect(ui.popupExceptionsPushButton,
	  SIGNAL(clicked(void)),
	  dooble::s_popupsWindow,
	  SLOT(slotShow(void)));
  connect(ui.thirdPartyExceptionsPushButton,
	  SIGNAL(clicked(void)),
	  dooble::s_adBlockWindow,
	  SLOT(slotShow(void)));
  connect(ui.cookiesExceptionsPushButton,
	  SIGNAL(clicked(void)),
	  dooble::s_cookiesBlockWindow,
	  SLOT(slotShow(void)));
  connect(ui.httpRedirectExceptionsPushButton,
	  SIGNAL(clicked(void)),
	  dooble::s_httpRedirectWindow,
	  SLOT(slotShow(void)));
  connect(ui.httpReferrerExceptionsPushButton,
	  SIGNAL(clicked(void)),
	  dooble::s_httpReferrerWindow,
	  SLOT(slotShow(void)));
  connect(ui.javaScriptExceptionsPushButton,
	  SIGNAL(clicked(void)),
	  dooble::s_javaScriptExceptionsWindow,
	  SLOT(slotShow(void)));
  connect(ui.rememberClosedTabsCheckBox,
	  SIGNAL(clicked(bool)),
	  ui.rememberClosedTabsSpinBox,
	  SLOT(setEnabled(bool)));
  connect(ui.iconSetComboBox,
	  SIGNAL(currentIndexChanged(int)),
	  this,
	  SLOT(slotIconsPreview(void)));
  connect(this,
	  SIGNAL(iconsChanged(void)),
	  this,
	  SLOT(slotSetIcons(void)));

  foreach(dsettingshomelinewidget *textEdit,
	  findChildren<dsettingshomelinewidget *> ())
    connect(this,
	    SIGNAL(iconsChanged(void)),
	    textEdit,
	    SLOT(slotSetIcons(void)));

  /*
  ** As long as the tab order of the buttons is correct, this trick
  ** will always work.
  */

  int idx = 0;

  foreach(QToolButton *button,
	  ui.tabScrollArea->findChildren<QToolButton *> ())
    {
      connect(button,
	      SIGNAL(clicked(bool)),
	      this,
	      SLOT(slotChangePage(bool)));
      button->setProperty("page", idx);
      idx += 1;
    }

  QFileInfo fileInfo
    (dooble::s_settings.value("settingsWindow/myRetrievedFiles",
			      "").toString());

  if(!(fileInfo.isReadable() && fileInfo.isWritable()))
    {
      QString path = QDesktopServices::storageLocation
	(QDesktopServices::DesktopLocation);

      if(path.isEmpty() || !QFile::exists(path) ||
	 !QFileInfo(path).isReadable())
	path = QDir::homePath();

      settings.setValue("settingsWindow/myRetrievedFiles", path);
      dooble::s_settings["settingsWindow/myRetrievedFiles"] = path;
    }

  ui.myRetrievedFilesLineEdit->setText
    (dooble::s_settings.value("settingsWindow/myRetrievedFiles").toString());
  slotSetIcons();
  slotCookieTimerTimeChanged(0);
}

dsettings::~dsettings()
{
}

void dsettings::exec(dooble *parent)
{
  m_parentDooble = parent;
  m_previousIconSet.clear();

  /*
  ** A little trick to force state changes.
  */

  ui.browsingProxyManualRadio->setChecked(true);
  ui.downloadProxyManualRadio->setChecked(true);
  ui.browsingProxyNoneRadio->setChecked(true);
  ui.downloadProxyNoneRadio->setChecked(true);
  ui.url01lineEdit->setText
    (dooble::s_settings.value("settingsWindow/url1", "").toString().trimmed());
  ui.url02lineEdit->setText
    (dooble::s_settings.value("settingsWindow/url2", "").toString().trimmed());
  ui.url03lineEdit->setText
    (dooble::s_settings.value("settingsWindow/url3", "").toString().trimmed());
  ui.url04lineEdit->setText
    (dooble::s_settings.value("settingsWindow/url4", "").toString().trimmed());
  ui.url05lineEdit->setText
    (dooble::s_settings.value("settingsWindow/url5", "").toString().trimmed());
  ui.url06lineEdit->setText
    (dooble::s_settings.value("settingsWindow/url6", "").toString().trimmed());
  ui.url07lineEdit->setText
    (dooble::s_settings.value("settingsWindow/url7", "").toString().trimmed());
  ui.url08lineEdit->setText
    (dooble::s_settings.value("settingsWindow/url8", "").toString().trimmed());
  ui.url09lineEdit->setText
    (dooble::s_settings.value("settingsWindow/url9", "").toString().trimmed());
  ui.url10lineEdit->setText
    (dooble::s_settings.value("settingsWindow/url10",
			      "").toString().trimmed());
  ui.url11lineEdit->setText
    (dooble::s_settings.value("settingsWindow/url11",
			      "").toString().trimmed());

  QLocale locale;

  ui.url12lineEdit->setText
    (dooble::s_settings.value
     ("settingsWindow/url12_1",
      QString("qrc:/search_%1.html").arg(locale.name().left(2))).toString().
     trimmed());
  ui.p2pLineEdit->setText
    (dooble::s_settings.value("settingsWindow/p2pUrl",
			      "about: blank").toString().trimmed());
  ui.displayDesktopCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/displayDesktopCheckBox",
			      false).toBool());
  ui.proceedToNewTabCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/proceedToNewTab",
			      true).toBool());
  ui.openInNewTabsCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/openInNewTab", true).toBool());
  ui.addTabWithDoubleClick->setChecked
    (dooble::s_settings.value("settingsWindow/addTabWithDoubleClick",
			      true).toBool());
  ui.webPluginsCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/enableWebPlugins",
			      true).toBool());
#ifdef Q_WS_X11
  ui.webPluginsCheckBox->setText(tr("Web Plugins (Unstable)"));
#endif
  ui.browsingHttpProxyGroupBox->setChecked
    (dooble::s_settings.value("settingsWindow/httpBrowsingProxyEnabled",
			      false).toBool());
  ui.downloadFtpProxyGroupBox->setChecked
    (dooble::s_settings.value("settingsWindow/ftpDownloadProxyEnabled",
			      false).toBool());
  ui.browsingFtpProxyGroupBox->setChecked
    (dooble::s_settings.value("settingsWindow/ftpBrowsingProxyEnabled",
			      false).toBool());
  ui.downloadHttpProxyGroupBox->setChecked
    (dooble::s_settings.value("settingsWindow/httpDownloadProxyEnabled",
			      false).toBool());
  ui.browsingHttpProxyType->setCurrentIndex
    (ui.browsingHttpProxyType->findText
     (dooble::s_settings.value("settingsWindow/httpBrowsingProxyType",
			       "Socks5").toString().trimmed()));
  ui.browsingHttpProxyHostName->setText
    (dooble::s_settings.value("settingsWindow/httpBrowsingProxyHost",
			      "").toString().trimmed());
  ui.browsingHttpProxyPort->setValue
    (dooble::s_settings.value("settingsWindow/httpBrowsingProxyPort",
			      1080).toInt());
  ui.browsingHttpProxyUserName->setText
    (dooble::s_settings.value("settingsWindow/httpBrowsingProxyUser",
			      "").toString().trimmed());
  ui.browsingHttpProxyPassword->setText
    (dooble::s_settings.value
     ("settingsWindow/httpBrowsingProxyPassword",
      "").toString().trimmed());
  ui.browsingI2pProxyGroupBox->setChecked
    (dooble::s_settings.value("settingsWindow/i2pBrowsingProxyEnabled",
			      true).toBool());
  ui.browsingI2pProxyType->setCurrentIndex
    (ui.browsingI2pProxyType->findText
     (dooble::s_settings.value("settingsWindow/i2pBrowsingProxyType",
			       "Http").toString().trimmed()));
  ui.browsingI2pProxyHostName->setText(dooble::s_settings.value
				       ("settingsWindow/i2pBrowsingProxyHost",
					"127.0.0.1").toString().trimmed());
  ui.browsingI2pProxyPort->setValue(dooble::s_settings.value
				    ("settingsWindow/i2pBrowsingProxyPort",
				     4444).toInt());
  ui.downloadI2pProxyGroupBox->setChecked
    (dooble::s_settings.value("settingsWindow/i2pDownloadProxyEnabled",
			      true).toBool());
  ui.downloadI2pProxyType->setCurrentIndex
    (ui.downloadI2pProxyType->findText
     (dooble::s_settings.value("settingsWindow/i2pDownloadProxyType",
			       "Http").toString().trimmed()));
  ui.downloadI2pProxyHostName->setText(dooble::s_settings.value
				       ("settingsWindow/i2pDownloadProxyHost",
					"127.0.0.1").toString().trimmed());
  ui.downloadI2pProxyPort->setValue(dooble::s_settings.value
				    ("settingsWindow/i2pDownloadProxyPort",
				     4444).toInt());
  ui.downloadFtpProxyHostName->setText
    (dooble::s_settings.value("settingsWindow/ftpDownloadProxyHost",
			      "").toString().trimmed());
  ui.downloadFtpProxyPort->setValue
    (dooble::s_settings.value("settingsWindow/ftpDownloadProxyPort",
			      21).toInt());
  ui.browsingFtpProxyHostName->setText
    (dooble::s_settings.value("settingsWindow/ftpBrowsingProxyHost",
			      "").toString().trimmed());
  ui.browsingFtpProxyPort->setValue
    (dooble::s_settings.value("settingsWindow/ftpBrowsingProxyPort",
			      21).toInt());
  ui.downloadHttpProxyType->setCurrentIndex
    (ui.downloadHttpProxyType->findText
     (dooble::s_settings.value("settingsWindow/httpDownloadProxyType",
			       "Socks5").toString().trimmed()));
  ui.downloadHttpProxyHostName->setText
    (dooble::s_settings.value("settingsWindow/httpDownloadProxyHost",
			      "").toString().trimmed());
  ui.downloadHttpProxyPort->setValue
    (dooble::s_settings.value("settingsWindow/httpDownloadProxyPort",
			      1080).toInt());
  ui.downloadHttpProxyUserName->setText
    (dooble::s_settings.value("settingsWindow/httpDownloadProxyUser",
			      "").toString().trimmed());
  ui.downloadHttpProxyPassword->setText
    (dooble::s_settings.value("settingsWindow/httpDownloadProxyPassword",
			      "").toString().trimmed());
  ui.automaticallyLoadImagesCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/automaticallyLoadImages",
			      true).toBool());
  ui.thirdPartyContentCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/blockThirdPartyContent",
			      false).toBool());
  ui.openUserWindowsInNewProcessesCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/openUserWindowsInNewProcesses",
			      false).toBool());
  ui.closeViaMiddleCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/closeViaMiddle", true).toBool());
  ui.suppressHttpRedirectCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/suppressHttpRedirect", true).
     toBool());
  ui.suppressHttpReferrerCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/suppressHttpReferrer", true).
     toBool());
  ui.xssAuditingCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/xssAuditingEnabled", false).
     toBool());
  ui.appendNewTabsCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/appendNewTabs", false).
     toBool());
  ui.displayIpAddressCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/displayIpAddress", false).
     toBool());

  QStringList allKeys(dooble::s_settings.keys());
  QMap<QString, QString> itemsMap;

  for(int i = 0; i < allKeys.size(); i++)
    if(allKeys[i].contains("iconSet"))
      {
	if(!QFileInfo(dooble::s_settings.value
		      (allKeys[i]).toString()).exists())
	  {
	    /*
	    ** Remove the setting from the configuration file.
	    */

	    dooble::s_settings.remove(allKeys[i]);
	    continue;
	  }

	QSettings settings
	  (dooble::s_settings.value(allKeys[i]).toString(),
	   QSettings::IniFormat);

	settings.beginGroup("Description");
	itemsMap[settings.value("Name").toString()] =
	  dooble::s_settings.value(allKeys[i]).toString();
      }

  allKeys.clear();
  ui.iconSetComboBox->blockSignals(true);
  ui.iconSetComboBox->clear();
  ui.iconSetComboBox->blockSignals(false);

  QSettings settings
    (dooble::s_settings.value("iconSet").toString(), QSettings::IniFormat);

  settings.beginGroup("Description");
  ui.iconSetComboBox->blockSignals(true);

  for(int i = 0; i < itemsMap.size(); i++)
    {
      ui.iconSetComboBox->addItem(itemsMap.keys()[i],
				  itemsMap.values()[i]);

      if(settings.value("Name").toString() == itemsMap.keys()[i])
	ui.iconSetComboBox->setCurrentIndex(i);
    }

  ui.iconSetComboBox->blockSignals(false);
  itemsMap.clear();
  ui.javaCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/javaEnabled", false).toBool());
  ui.javascriptGroupBox->setChecked
    (dooble::s_settings.value("settingsWindow/javascriptEnabled", true).
     toBool());
  ui.jsAllowNewWindowsCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/javascriptAllowNewWindows",
			      false).toBool());
  ui.cookieGroupBox->setChecked
    (dooble::s_settings.value("settingsWindow/cookiesEnabled", true).toBool());
  ui.blockPopupsCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/blockPopups", false).toBool());
  ui.languageComboBox->clear();

  QFont font;

  if(font.fromString(dooble::s_settings.value("settingsWindow/globalFont", "").
		     toString()) &&
     !font.family().isEmpty())
    {
      if(ui.fontComboBox->findText(font.family()) > -1)
	ui.fontComboBox->setCurrentIndex
	  (ui.fontComboBox->findText(font.family()));
      else
	ui.fontComboBox->setCurrentIndex(0);

      updateFontWidgets(font.family(), ui.fontSizeComboBox);

      QString globalFontSize(QString::number(font.pointSize()));

      if(ui.fontSizeComboBox->findText(globalFontSize) > -1)
	ui.fontSizeComboBox->setCurrentIndex
	  (ui.fontSizeComboBox->findText(globalFontSize));
      else
	ui.fontSizeComboBox->setCurrentIndex(0);
    }
  else
    {
      QString text("");
      QString fontSize("");

#ifdef Q_WS_MAC
      text = "Times";
      fontSize = "10";
#elif defined Q_WS_WIN
      text = "Arial";
      fontSize = "10";
#else
      text = "Serif";
      fontSize = "9";
#endif

      if(ui.fontComboBox->findText(text) > -1)
	ui.fontComboBox->setCurrentIndex
	  (ui.fontComboBox->findText(text));
      else
	ui.fontComboBox->setCurrentIndex(0);

      updateFontWidgets(ui.fontComboBox->currentText(),
			ui.fontSizeComboBox);

      if(ui.fontSizeComboBox->findText(fontSize) > -1)
	ui.fontSizeComboBox->setCurrentIndex
	  (ui.fontSizeComboBox->findText(fontSize));
      else
	ui.fontSizeComboBox->setCurrentIndex(0);
    }

  if(font.fromString(dooble::s_settings.value
		     ("settingsWindow/standardWebFont", "").
		     toString()) &&
     !font.family().isEmpty())
    {
      if(ui.defaultFontCombinationBox->findText(font.family()) > -1)
	ui.defaultFontCombinationBox->setCurrentIndex
	  (ui.defaultFontCombinationBox->findText(font.family()));
      else
	ui.defaultFontCombinationBox->setCurrentIndex(0);

      updateFontWidgets(font.family(), ui.defaultFontSizeCombinationBox);

      QString fontSize(QString::number(font.pointSize()));

      if(ui.defaultFontSizeCombinationBox->findText(fontSize) > -1)
	ui.defaultFontSizeCombinationBox->setCurrentIndex
	  (ui.defaultFontSizeCombinationBox->findText(fontSize));
      else
	ui.defaultFontSizeCombinationBox->setCurrentIndex(0);
    }
  else
    {
      QString text("");
      QString fontSize("");

#ifdef Q_WS_MAC
      text = "Times";
      fontSize = "16";
#elif defined Q_WS_WIN
      text = "Serif";
      fontSize = "10";
#else
      text = "Serif";
      fontSize = "16";
#endif

      if(ui.defaultFontCombinationBox->findText(text) > -1)
	ui.defaultFontCombinationBox->setCurrentIndex
	  (ui.defaultFontCombinationBox->findText(text));
      else
	ui.defaultFontCombinationBox->setCurrentIndex(0);

      updateFontWidgets(ui.defaultFontCombinationBox->currentText(),
			ui.defaultFontSizeCombinationBox);

      if(ui.defaultFontSizeCombinationBox->findText(fontSize) > -1)
	ui.defaultFontSizeCombinationBox->setCurrentIndex
	  (ui.defaultFontSizeCombinationBox->findText(fontSize));
      else
	ui.defaultFontSizeCombinationBox->setCurrentIndex(0);
    }

  if(font.fromString
     (dooble::s_settings.value("settingsWindow/cursiveWebFont", "").
      toString()) &&
     !font.family().isEmpty())
    {
      if(ui.cursiveFontCombinationBox->findText(font.family()) > -1)
	ui.cursiveFontCombinationBox->setCurrentIndex
	  (ui.cursiveFontCombinationBox->findText(font.family()));
      else
	ui.cursiveFontCombinationBox->setCurrentIndex(0);
    }
  else
    {
      if(ui.cursiveFontCombinationBox->
	 findText("Serif") > -1)
	ui.cursiveFontCombinationBox->setCurrentIndex
	  (ui.cursiveFontCombinationBox->
	   findText("Serif"));
      else
	ui.cursiveFontCombinationBox->setCurrentIndex(0);
    }

  if(font.fromString
     (dooble::s_settings.value("settingsWindow/fantasyWebFont", "").
      toString()) &&
     !font.family().isEmpty())
    {
      if(ui.fantasyFontCombinationBox->findText(font.family()) > -1)
	ui.fantasyFontCombinationBox->setCurrentIndex
	  (ui.fantasyFontCombinationBox->findText(font.family()));
      else
	ui.fantasyFontCombinationBox->setCurrentIndex(0);
    }
  else
    {
      if(ui.fantasyFontCombinationBox->
	 findText("Serif") > -1)
	ui.fantasyFontCombinationBox->setCurrentIndex
	  (ui.fantasyFontCombinationBox->
	   findText("Serif"));
      else
	ui.fantasyFontCombinationBox->setCurrentIndex(0);
    }

  if(font.fromString
     (dooble::s_settings.value("settingsWindow/fixedWebFont", "").
      toString()) &&
     !font.family().isEmpty())
    {
      if(ui.fixedFontCombinationBox->findText(font.family()) > -1)
	ui.fixedFontCombinationBox->setCurrentIndex
	  (ui.fixedFontCombinationBox->findText(font.family()));
      else
	ui.fixedFontCombinationBox->setCurrentIndex(0);

      updateFontWidgets(font.family(), ui.fixedFontSizeCombinationBox);

      QString fontSize(QString::number(font.pointSize()));

      if(ui.fixedFontSizeCombinationBox->findText(fontSize) > -1)
	ui.fixedFontSizeCombinationBox->setCurrentIndex
	  (ui.fixedFontSizeCombinationBox->findText(fontSize));
      else
	ui.fixedFontSizeCombinationBox->setCurrentIndex(0);
    }
  else
    {
      int index = -1;
      QString text("");
      QString fontSize("");

#ifdef Q_WS_MAC
      text = "Courier";
      fontSize = "13";
#elif defined Q_WS_WIN
      text = "Courier New";
      fontSize = "16";
#else
      text = "Courier";
      fontSize = "10";
#endif

      if((index = ui.fixedFontCombinationBox->
	  findText(text, Qt::MatchContains)) > -1)
	ui.fixedFontCombinationBox->setCurrentIndex(index);
      else
	ui.fixedFontCombinationBox->setCurrentIndex(0);

      updateFontWidgets(ui.fixedFontCombinationBox->currentText(),
			ui.fixedFontSizeCombinationBox);

      if(ui.fixedFontSizeCombinationBox->findText(fontSize) > -1)
	ui.fixedFontSizeCombinationBox->setCurrentIndex
	  (ui.fixedFontSizeCombinationBox->findText(fontSize));
      else
	ui.fixedFontSizeCombinationBox->setCurrentIndex(0);
    }

  if(font.fromString
     (dooble::s_settings.value("settingsWindow/sansSerifWebFont", "").
      toString()) &&
     !font.family().isEmpty())
    {
      if(ui.sansSerifFontCombinationBox->findText(font.family()) > -1)
	ui.sansSerifFontCombinationBox->setCurrentIndex
	  (ui.sansSerifFontCombinationBox->findText(font.family()));
      else
	ui.sansSerifFontCombinationBox->setCurrentIndex(0);
    }
  else
    {
      QString text("");

#ifdef Q_WS_MAC
      text = "Helvetica";
#elif defined Q_WS_WIN
      text = "Arial";
#else
      text = "Sans Serif";
#endif

      if(ui.sansSerifFontCombinationBox->
	 findText(text) > -1)
	ui.sansSerifFontCombinationBox->setCurrentIndex
	  (ui.sansSerifFontCombinationBox->
	   findText(text));
      else
	ui.sansSerifFontCombinationBox->setCurrentIndex(0);
    }

  if(font.fromString
     (dooble::s_settings.value("settingsWindow/serifWebFont", "").
      toString()) && !font.family().isEmpty())
    {
      if(ui.serifFontCombinationBox->findText(font.family()) > -1)
	ui.serifFontCombinationBox->setCurrentIndex
	  (ui.serifFontCombinationBox->findText(font.family()));
      else
	ui.serifFontCombinationBox->setCurrentIndex(0);
    }
  else
    {
      QString text("");

#ifdef Q_WS_MAC
      text = "Times";
#elif defined Q_WS_WIN
      text = "Times New Roman";
#else
      text = "Serif";
#endif

      if(ui.serifFontCombinationBox->
	 findText(text) > -1)
	ui.serifFontCombinationBox->setCurrentIndex
	  (ui.serifFontCombinationBox->findText(text));
      else
	ui.serifFontCombinationBox->setCurrentIndex(0);
    }

  QString size = dooble::s_settings.value
    ("settingsWindow/minimumWebFontSize", "9").toString();

  if(ui.minimumFontSizeCombinationBox->findText(size) > -1)
    ui.minimumFontSizeCombinationBox->setCurrentIndex
      (ui.minimumFontSizeCombinationBox->findText(size));
  else
    ui.minimumFontSizeCombinationBox->setCurrentIndex(0);

  QList<int> mibs = QTextCodec::availableMibs();
  QStringList codecs;

  for(int i = 0; i < mibs.size(); i++)
    {
      QString codec = QLatin1String
	(QTextCodec::codecForMib(mibs.at(i))->name());

      codecs.append(codec);
    }

  codecs.sort();
  ui.encodingCombinationBox->addItems(codecs);
  codecs.clear();

  QString text(dooble::s_settings.value("settingsWindow/characterEncoding",
					"").toString());

  if(ui.encodingCombinationBox->findText(text) > -1)
    ui.encodingCombinationBox->setCurrentIndex
      (ui.encodingCombinationBox->findText(text));
  else
    {
      text = QWebSettings::globalSettings()->defaultTextEncoding();

      for(int i = 0; i < ui.encodingCombinationBox->count(); i++)
	{
	  QString str(ui.encodingCombinationBox->itemText(i));

	  if(str.toLower() == text.toLower())
	    {
	      ui.encodingCombinationBox->setCurrentIndex(i);
	      break;
	    }
	}
    }

  ui.jsAcceptGeometryRequestsCheckBox->setChecked
    (dooble::s_settings.value
     ("settingsWindow/javascriptAcceptGeometryChangeRequests",
      false).toBool());
  ui.historyCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/rememberHistory",
			      true).toBool());
  ui.downloadCheckBox->setChecked
    (dooble::s_settings.value
     ("settingsWindow/rememberDownloads", true).toBool());
  ui.historySpinBox->setValue
    (dooble::s_settings.value("settingsWindow/historyDays", 8).toInt());
  ui.historySpinBox->setEnabled(ui.historyCheckBox->isChecked());
  ui.jsHideMenuBarCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/javascriptAllowMenuBarHiding",
			      false).toBool());
  ui.jsHideStatusBarCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/javascriptAllowStatusBarHiding",
			      false).toBool());
  ui.jsHideToolBarCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/javascriptAllowToolBarHiding",
			      false).toBool());
  ui.alwaysShowTabBarCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/alwaysShowTabBar",
			      true).toBool());
#if QT_VERSION >= 0x040700
  ui.spatialNavigationCheckBox->setEnabled(true);
  ui.xssAuditingCheckBox->setEnabled(true);
#else
  ui.spatialNavigationCheckBox->setEnabled(false);
  ui.xssAuditingCheckBox->setEnabled(false);
#endif
  ui.spatialNavigationCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/spatialNavigation",
			      false).toBool());
  ui.closeTabWarningCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/warnBeforeClosingModifiedTab",
			      false).toBool());
  ui.leaveModifiedTabCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/warnBeforeLeavingModifiedTab",
			      false).toBool());

  if(dooble::s_settings.value("settingsWindow/cookiesShouldBe",
			      0).toInt() == 0)
    ui.cookieComboBox->setCurrentIndex(0);
  else if(dooble::s_settings.value("settingsWindow/cookiesShouldBe",
				   0).toInt() == 1)
    ui.cookieComboBox->setCurrentIndex(1);
  else if(dooble::s_settings.value("settingsWindow/cookiesShouldBe",
				   0).toInt() == 2)
    ui.cookieComboBox->setCurrentIndex(2);
  else
    ui.cookieComboBox->setCurrentIndex(0);

  ui.languageComboBox->addItem(locale.languageToString(locale.language()));

  QToolButton *panelButton = 0;

  foreach(QToolButton *button,
	  ui.tabScrollArea->findChildren<QToolButton *> ())
    if(button->property("page").toInt() ==
       dooble::s_settings.value("settingsWindow/currentPage", 0).toInt())
      {
	panelButton = button;
	button->click();
	button->setFocus();
	break;
      }

  if(dooble::s_settings.value("settingsWindow/cookieTimerEnabled",
			      false).toBool())
    if(!(ui.cookieTimerCheckBox->checkState() == Qt::Checked))
      {
	ui.cookieTimerCheckBox->click();
	ui.cookieTimerCheckBox->setChecked(true);
      }

  ui.cookieTimerSpinBox->setValue
    (dooble::s_settings.value("settingsWindow/cookieTimerInterval",
			      1).toInt());
  ui.cookieTimerComboBox->setCurrentIndex
    (dooble::s_settings.value("settingsWindow/cookieTimerUnit", 0).toInt());
  ui.textSizeMultiplierSpinBox->setValue
    (dooble::s_settings.value("settingsWindow/textSizeMultiplier",
			      1.00).toFloat());
  ui.rememberClosedTabsCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/rememberClosedTabs", false).
     toBool());
  ui.rememberClosedTabsSpinBox->setEnabled
    (ui.rememberClosedTabsCheckBox->isChecked());
  ui.rememberClosedTabsSpinBox->setValue
    (dooble::s_settings.value("settingsWindow/rememberClosedTabsCount", 1).
     toInt());
  ui.closeDownloadsCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/closeDownloads",
			      false).toBool());
  ui.reencodeBookmarksCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/reencodeBookmarks", false).
     toBool());
  ui.reencodeCookiesCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/reencodeCookies", false).
     toBool());
  ui.reencodeDownloadsCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/reencodeDownloads", false).
     toBool());
  ui.reencodeExceptionsCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/reencodeExceptions", false).
     toBool());
  ui.reencodeHistoryCheckBox->setChecked
    (dooble::s_settings.value("settingsWindow/reencodeHistory", false).
     toBool());

  if(!dooble::s_settings.value("settingsWindow/hashType", "").
     toString().isEmpty() &&
     !dooble::s_settings.value("settingsWindow/passphraseHash", "").
     toString().isEmpty())
    {
      if(dmisc::isHashTypeSupported(dooble::s_settings.
				    value("settingsWindow/hashType").
				    toString()))
	{
	  QString hash("");
	  QString hashType
	    (dooble::s_settings.value("settingsWindow/hashType").
	     toString());
	  QString passphraseHash
	    (dooble::s_settings.value("settingsWindow/passphraseHash").
	     toString());

	  hash = dmisc::passphraseHash(hashType);

	  if(hash != passphraseHash)
	    {
	      ui.passphraseGroupBox->setEnabled(false);
	      ui.changePassphrasePushButton->setEnabled(false);
	    }
	  else
	    {
	      ui.passphraseGroupBox->setEnabled(false);
	      ui.changePassphrasePushButton->setEnabled(true);
	    }
	}
      else
	{
	  ui.passphraseGroupBox->setEnabled(true);
	  ui.changePassphrasePushButton->setEnabled(false);
	}
    }
  else
    {
      ui.passphraseGroupBox->setEnabled(true);
      ui.changePassphrasePushButton->setEnabled(false);
    }

  QString str1(""), str2("");

  if(dooble::s_settings.contains("cipherType"))
    str1 = dooble::s_settings.value("cipherType").toString();
  else
    str1 = tr("None");

  str2 = dmisc::findStrongestCipherType();

  if(str2.isEmpty())
    str2 = tr("None");

  ui.cipherTypeLabel->setText
    ("<html>" + tr("Strongest Available Cipher Type: ") +
     "<b>" + str2 + "</b><br>" +
     tr("Current Cipher Type: ") +
     "<b>" + str1 + "</b></html>");

  if(dooble::s_settings.contains("settingsWindow/hashType"))
    str1 = dooble::s_settings.value("settingsWindow/hashType").toString();
  else
    str1 = tr("None");

  str2 = dmisc::findStrongestHashType();

  if(str2.isEmpty())
    str2 = tr("None");

  ui.passphraseHashTypeLabel->setText
    ("<html>" + tr("Strongest Available Hash Type: ") +
     "<b>" + str2 + "</b><br>" +
     tr("Current Hash Type: ") +
     "<b>" + str1 + "</b></html>");
  str1 = dooble::s_settings.value("settingsWindow/browsingProxySetting",
				  "none").
    toString();

  if(str1 == "none")
    ui.browsingProxyNoneRadio->click();
  else if(str1 == "manual")
    ui.browsingProxyManualRadio->click();
  else
    ui.browsingProxySystemRadio->click();

  str1 = dooble::s_settings.value("settingsWindow/downloadProxySetting",
				  "none").
    toString();

  if(str1 == "none")
    ui.downloadProxyNoneRadio->click();
  else if(str1 == "manual")
    ui.downloadProxyManualRadio->click();
  else
    ui.downloadProxySystemRadio->click();

#ifndef Q_WS_MAC
  int l_height = 0;

  if(m_parentDooble)
    l_height = m_parentDooble->height() -
      0.30 * m_parentDooble->height();
  else
    l_height = sizeHint().height();

  resize(sizeHint().width() + 0.30 * sizeHint().width(),
  	 l_height);
#endif

  int X = 0;
  int Y = 0;
  QPoint p(0, 0);

  p = m_parentDooble->pos();

  if(m_parentDooble->width() >= width())
    X = p.x() + (m_parentDooble->width() - width()) / 2;
  else
    X = p.x() - (width() - m_parentDooble->width()) / 2;

  if(m_parentDooble->height() >= height())
    Y = p.y() + (m_parentDooble->height() - height()) / 2;
  else
    Y = p.y() - (height() - m_parentDooble->height()) / 2;

  move(X, Y);
  showNormal();
  raise();

  if(panelButton)
    panelButton->setFocus();
}

void dsettings::slotClicked(QAbstractButton *button)
{
  if(ui.buttonBox->buttonRole(button) == QDialogButtonBox::AcceptRole)
    {
      if(ui.passphraseGroupBox->isEnabled())
	{
	  QString pass1(ui.pass1LineEdit->text().trimmed());
	  QString pass2(ui.pass2LineEdit->text().trimmed());

	  if(pass1.isEmpty() && pass2.isEmpty())
	    {
	      // Do nothing.
	    }
	  else if(pass1 != pass2)
	    {
	      foreach(QToolButton *button,
		      ui.tabScrollArea->findChildren<QToolButton *> ())
		if(button->property("page").toInt() == 4)
		  button->click();

	      QSettings settings(dooble::s_settings.value("iconSet").
				 toString(),
				 QSettings::IniFormat);
	      QMessageBox mb(QMessageBox::Critical,
			     tr("Dooble Web Browser: Error"),
			     tr("The passphrases do not match."),
			     QMessageBox::Cancel,
			     this);

#ifdef Q_WS_MAC
	      mb.setAttribute(Qt::WA_MacMetalStyle, true);
#endif
	      mb.setWindowIcon
		(QIcon(settings.value("mainWindow/windowIcon").toString()));

	      for(int i = 0; i < mb.buttons().size(); i++)
		mb.buttons().at(i)->setIcon
		  (QIcon(settings.value("cancelButtonIcon").toString()));

	      mb.exec();
	      return;
	    }
	  else if(pass1.length() < 12)
	    {
	      foreach(QToolButton *button,
		      ui.tabScrollArea->findChildren<QToolButton *> ())
		if(button->property("page").toInt() == 4)
		  button->click();

	      QSettings settings(dooble::s_settings.value("iconSet").
				 toString(),
				 QSettings::IniFormat);
	      QMessageBox mb(QMessageBox::Critical,
			     tr("Dooble Web Browser: Error"),
			     tr("The passphrase must be at least "
				"twelve characters "
				"long."),
			     QMessageBox::Cancel,
			     this);

#ifdef Q_WS_MAC
	      mb.setAttribute(Qt::WA_MacMetalStyle, true);
#endif
	      mb.setWindowIcon
		(QIcon(settings.value("mainWindow/windowIcon").toString()));

	      for(int i = 0; i < mb.buttons().size(); i++)
		mb.buttons().at(i)->setIcon
		  (QIcon(settings.value("cancelButtonIcon").toString()));

	      mb.exec();
	      return;
	    }

	  if(!pass1.isEmpty())
	    dmisc::setCipherPassphrase(pass1);

	  /*
	  ** Clear the passphrase containers immediately.
	  */

	  pass1.clear();
	  pass2.clear();
	  ui.pass1LineEdit->clear();
	  ui.pass2LineEdit->clear();
	}

      m_previousIconSet.clear();

      QSettings settings;

      settings.setValue
	("iconSet",
	 ui.iconSetComboBox->itemData(ui.iconSetComboBox->currentIndex()).
	 toString());
      dooble::s_settings["iconSet"] =
	ui.iconSetComboBox->itemData(ui.iconSetComboBox->currentIndex()).
	toString();
      settings.setValue("settingsWindow/url1",
			ui.url01lineEdit->text().trimmed());
      settings.setValue("settingsWindow/url2",
			ui.url02lineEdit->text().trimmed());
      settings.setValue("settingsWindow/url3",
			ui.url03lineEdit->text().trimmed());
      settings.setValue("settingsWindow/url4",
			ui.url04lineEdit->text().trimmed());
      settings.setValue("settingsWindow/url5",
			ui.url05lineEdit->text().trimmed());
      settings.setValue("settingsWindow/url6",
			ui.url06lineEdit->text().trimmed());
      settings.setValue("settingsWindow/url7",
			ui.url07lineEdit->text().trimmed());
      settings.setValue("settingsWindow/url8",
			ui.url08lineEdit->text().trimmed());
      settings.setValue("settingsWindow/url9",
			ui.url09lineEdit->text().trimmed());
      settings.setValue("settingsWindow/url10",
			ui.url10lineEdit->text().trimmed());
      settings.setValue("settingsWindow/url11",
			ui.url11lineEdit->text().trimmed());
      settings.setValue("settingsWindow/url12_1",
			ui.url12lineEdit->text().trimmed());
      settings.setValue("settingsWindow/myRetrievedFiles",
			ui.myRetrievedFilesLineEdit->text().trimmed());
      settings.setValue("settingsWindow/p2pUrl",
			ui.p2pLineEdit->text().trimmed());
      settings.setValue("settingsWindow/javaEnabled",
			ui.javaCheckBox->isChecked());
      settings.setValue("settingsWindow/javascriptEnabled",
			ui.javascriptGroupBox->isChecked());
      settings.setValue("settingsWindow/javascriptAllowNewWindows",
			ui.jsAllowNewWindowsCheckBox->isChecked());
      settings.setValue("settingsWindow/displayDesktopCheckBox",
			ui.displayDesktopCheckBox->isChecked());
      settings.setValue("settingsWindow/proceedToNewTab",
			ui.proceedToNewTabCheckBox->isChecked());
      settings.setValue("settingsWindow/openInNewTab",
			ui.openInNewTabsCheckBox->isChecked());
      settings.setValue("settingsWindow/httpBrowsingProxyEnabled",
			ui.browsingHttpProxyGroupBox->isChecked());
      settings.setValue("settingsWindow/httpBrowsingProxyType",
			ui.browsingHttpProxyType->currentText().trimmed());
      settings.setValue("settingsWindow/httpBrowsingProxyHost",
			ui.browsingHttpProxyHostName->text().trimmed());
      settings.setValue("settingsWindow/httpBrowsingProxyPort",
			ui.browsingHttpProxyPort->value());
      settings.setValue("settingsWindow/httpBrowsingProxyUser",
			ui.browsingHttpProxyUserName->text().trimmed());
      settings.setValue("settingsWindow/httpBrowsingProxyPassword",
			ui.browsingHttpProxyPassword->text().trimmed());
      settings.setValue("settingsWindow/i2pBrowsingProxyEnabled",
			ui.browsingI2pProxyGroupBox->isChecked());
      settings.setValue("settingsWindow/i2pBrowsingProxyType",
			ui.browsingI2pProxyType->currentText().trimmed());
      settings.setValue("settingsWindow/i2pBrowsingProxyHost",
			ui.browsingI2pProxyHostName->text().trimmed());
      settings.setValue("settingsWindow/i2pBrowsingProxyPort",
			ui.browsingI2pProxyPort->value());
      settings.setValue("settingsWindow/i2pDownloadProxyEnabled",
			ui.downloadI2pProxyGroupBox->isChecked());
      settings.setValue("settingsWindow/i2pDownloadProxyType",
			ui.downloadI2pProxyType->currentText().trimmed());
      settings.setValue("settingsWindow/i2pDownloadProxyHost",
			ui.downloadI2pProxyHostName->text().trimmed());
      settings.setValue("settingsWindow/i2pDownloadProxyPort",
			ui.downloadI2pProxyPort->value());
      settings.setValue("settingsWindow/ftpDownloadProxyEnabled",
			ui.downloadFtpProxyGroupBox->isChecked());
      settings.setValue("settingsWindow/ftpDownloadProxyHost",
			ui.downloadFtpProxyHostName->text().trimmed());
      settings.setValue("settingsWindow/ftpDownloadProxyPort",
			ui.downloadFtpProxyPort->value());
      settings.setValue("settingsWindow/ftpBrowsingProxyEnabled",
			ui.browsingFtpProxyGroupBox->isChecked());
      settings.setValue("settingsWindow/ftpBrowsingProxyHost",
			ui.browsingFtpProxyHostName->text().trimmed());
      settings.setValue("settingsWindow/ftpBrowsingProxyPort",
			ui.browsingFtpProxyPort->value());
      settings.setValue("settingsWindow/httpDownloadProxyEnabled",
			ui.downloadHttpProxyGroupBox->isChecked());
      settings.setValue("settingsWindow/httpDownloadProxyType",
			ui.downloadHttpProxyType->currentText().trimmed());
      settings.setValue("settingsWindow/httpDownloadProxyHost",
			ui.downloadHttpProxyHostName->text().trimmed());
      settings.setValue("settingsWindow/httpDownloadProxyPort",
			ui.downloadHttpProxyPort->value());
      settings.setValue("settingsWindow/httpDownloadProxyUser",
			ui.downloadHttpProxyUserName->text().trimmed());
      settings.setValue("settingsWindow/httpDownloadProxyPassword",
			ui.downloadHttpProxyPassword->text().trimmed());
      settings.setValue("settingsWindow/automaticallyLoadImages",
			ui.automaticallyLoadImagesCheckBox->isChecked());
      settings.setValue("settingsWindow/blockThirdPartyContent",
			ui.thirdPartyContentCheckBox->isChecked());
      settings.setValue("settingsWindow/enableWebPlugins",
			ui.webPluginsCheckBox->isChecked());
      settings.setValue("settingsWindow/addTabWithDoubleClick",
			ui.addTabWithDoubleClick->isChecked());
      settings.setValue("settingsWindow/openUserWindowsInNewProcesses",
			ui.openUserWindowsInNewProcessesCheckBox->
			isChecked());
      settings.setValue("settingsWindow/closeViaMiddle",
			ui.closeViaMiddleCheckBox->isChecked());
      settings.setValue("settingsWindow/cookiesEnabled",
			ui.cookieGroupBox->isChecked());
      settings.setValue("settingsWindow/blockPopups",
			ui.blockPopupsCheckBox->isChecked());
      settings.setValue("settingsWindow/cursiveWebFont",
			ui.cursiveFontCombinationBox->
			currentFont().toString());
      settings.setValue("settingsWindow/fantasyWebFont",
			ui.fantasyFontCombinationBox->
			currentFont().toString());
      settings.setValue("settingsWindow/sansSerifWebFont",
			ui.sansSerifFontCombinationBox->
			currentFont().toString());
      settings.setValue("settingsWindow/serifWebFont",
			ui.serifFontCombinationBox->
			currentFont().toString());
      settings.setValue("settingsWindow/minimumWebFontSize",
			ui.minimumFontSizeCombinationBox->
			currentText().toInt());
      settings.setValue("settingsWindow/characterEncoding",
			ui.encodingCombinationBox->
			currentText());
      settings.setValue
	("settingsWindow/javascriptAcceptGeometryChangeRequests",
	 ui.jsAcceptGeometryRequestsCheckBox->isChecked());
      settings.setValue("settingsWindow/rememberHistory",
			ui.historyCheckBox->isChecked());
      settings.setValue("settingsWindow/rememberDownloads",
			ui.downloadCheckBox->isChecked());
      settings.setValue("settingsWindow/historyDays",
			ui.historySpinBox->value());
      settings.setValue("settingsWindow/javascriptAllowMenuBarHiding",
			ui.jsHideMenuBarCheckBox->isChecked());
      settings.setValue("settingsWindow/javascriptAllowStatusBarHiding",
			ui.jsHideStatusBarCheckBox->isChecked());
      settings.setValue("settingsWindow/javascriptAllowToolBarHiding",
			ui.jsHideToolBarCheckBox->isChecked());
      settings.setValue("settingsWindow/alwaysShowTabBar",
			ui.alwaysShowTabBarCheckBox->isChecked());
      settings.setValue("settingsWindow/spatialNavigation",
			ui.spatialNavigationCheckBox->isChecked());
      settings.setValue("settingsWindow/cookiesShouldBe",
			ui.cookieComboBox->currentIndex());
      settings.setValue("settingsWindow/textSizeMultiplier",
			ui.textSizeMultiplierSpinBox->value());

      if(ui.browsingProxyManualRadio->isChecked())
	settings.setValue("settingsWindow/browsingProxySetting", "manual");
      else if(ui.browsingProxySystemRadio->isChecked())
	settings.setValue("settingsWindow/browsingProxySetting", "system");
      else
	settings.setValue("settingsWindow/browsingProxySetting", "none");

      if(ui.downloadProxyManualRadio->isChecked())
	settings.setValue("settingsWindow/downloadProxySetting", "manual");
      else if(ui.downloadProxySystemRadio->isChecked())
	settings.setValue("settingsWindow/downloadProxySetting", "system");
      else
	settings.setValue("settingsWindow/downloadProxySetting", "none");

      QFont font;

      font = ui.fixedFontCombinationBox->currentFont();
      font.setPointSize(ui.fixedFontSizeCombinationBox->
			currentText().toInt());
      font.setWeight(QFont::Normal);
      settings.setValue("settingsWindow/fixedWebFont", font.toString());
      font = ui.fontComboBox->currentFont();
      font.setPointSize(ui.fontSizeComboBox->currentText().toInt());
      font.setWeight(QFont::Normal);
      QApplication::setFont(font);
      settings.setValue("settingsWindow/globalFont", font.toString());
      font = ui.defaultFontCombinationBox->currentFont();
      font.setPointSize(ui.defaultFontSizeCombinationBox->
			currentText().toInt());
      font.setWeight(QFont::Normal);
      settings.setValue("settingsWindow/standardWebFont", font.toString());
      settings.setValue("settingsWindow/currentPage",
			ui.stackedWidget->currentIndex());
      settings.setValue("settingsWindow/cookieTimerEnabled",
			ui.cookieTimerCheckBox->isChecked());
      settings.setValue("settingsWindow/cookieTimerInterval",
			ui.cookieTimerSpinBox->value());
      settings.setValue("settingsWindow/cookieTimerUnit",
			ui.cookieTimerComboBox->currentIndex());
      settings.setValue("settingsWindow/closeDownloads",
			ui.closeDownloadsCheckBox->isChecked());
      settings.setValue("settingsWindow/suppressHttpRedirect",
			ui.suppressHttpRedirectCheckBox->isChecked());
      settings.setValue("settingsWindow/suppressHttpReferrer",
			ui.suppressHttpReferrerCheckBox->isChecked());
      settings.setValue("settingsWindow/xssAuditingEnabled",
			ui.xssAuditingCheckBox->isChecked());
      settings.setValue("settingsWindow/appendNewTabs",
			ui.appendNewTabsCheckBox->isChecked());
      settings.setValue("settingsWindow/displayIpAddress",
			ui.displayIpAddressCheckBox->isChecked());
      settings.setValue("settingsWindow/warnBeforeClosingModifiedTab",
			ui.closeTabWarningCheckBox->isChecked());
      settings.setValue("settingsWindow/warnBeforeLeavingModifiedTab",
			ui.leaveModifiedTabCheckBox->isChecked());
      settings.setValue("settingsWindow/rememberClosedTabs",
			ui.rememberClosedTabsCheckBox->isChecked());
      settings.setValue("settingsWindow/rememberClosedTabsCount",
			ui.rememberClosedTabsSpinBox->value());
      settings.setValue("settingsWindow/reencodeBookmarks",
			ui.reencodeBookmarksCheckBox->isChecked());
      settings.setValue("settingsWindow/reencodeCookies",
			ui.reencodeCookiesCheckBox->isChecked());
      settings.setValue("settingsWindow/reencodeDownloads",
			ui.reencodeDownloadsCheckBox->isChecked());
      settings.setValue("settingsWindow/reencodeExceptions",
			ui.reencodeExceptionsCheckBox->isChecked());
      settings.setValue("settingsWindow/reencodeHistory",
			ui.reencodeHistoryCheckBox->isChecked());

      if(ui.passphraseGroupBox->isEnabled() &&
	 dmisc::passphraseWasAuthenticated())
	{
	  QString hash("");
	  QString hashType(dmisc::findStrongestHashType());

	  hash = dmisc::passphraseHash(hashType);

	  if(!hash.isEmpty())
	    {
	      settings.setValue("settingsWindow/passphraseHash", hash);
	      settings.setValue("settingsWindow/hashType", hashType);

	      /*
	      ** We also need to update dooble::s_settings.
	      */

	      dooble::s_settings["settingsWindow/passphraseHash"] = hash;
	      dooble::s_settings["settingsWindow/hashType"] = hashType;

	      QProgressDialog progress(this);

	      progress.setModal(true);
#ifdef Q_WS_MAC
	      progress.setAttribute(Qt::WA_MacMetalStyle, true);
#endif
	      progress.setCancelButton(0);

	      /*
	      ** Perform re-encoding only if a passphrase was
	      ** previously set.
	      */

	      if(ui.reencodeCookiesCheckBox->isChecked())
		dooble::s_cookies->reencode(&progress);

	      if(ui.reencodeDownloadsCheckBox->isChecked())
		dooble::s_downloadWindow->reencode(&progress);

	      if(ui.reencodeExceptionsCheckBox->isChecked())
		{
		  progress.reset();
		  progress.setWindowTitle(tr("Dooble: Progress Dialog"));
		  progress.setLabelText
		    (tr("Re-encoding cookie exceptions information..."));
		  dooble::s_cookiesBlockWindow->reencode(&progress);
		  progress.reset();
		  progress.setLabelText
		    (tr("Re-encoding HTTP redirect exceptions "
			"information..."));
		  dooble::s_httpRedirectWindow->reencode(&progress);
		  progress.reset();
		  progress.setLabelText
		    (tr("Re-encoding HTTP referrer exceptions "
			"information..."));
		  dooble::s_httpReferrerWindow->reencode(&progress);
		  progress.reset();
		  progress.setLabelText
		    (tr("Re-encoding JavaScript exceptions "
			"information..."));
		  dooble::s_javaScriptExceptionsWindow->reencode
		    (&progress);
		  progress.reset();
		  progress.setLabelText
		    (tr("Re-encoding JavaScript popups exceptions "
			"information..."));
		  dooble::s_popupsWindow->reencode(&progress);
		  progress.reset();
		  progress.setLabelText
		    (tr("Re-encoding third-party exceptions "
			"information..."));
		  dooble::s_adBlockWindow->reencode(&progress);
		}

	      if(ui.reencodeHistoryCheckBox->isChecked())
		dooble::s_historyWindow->reencode(&progress);
	    }
	}

      QWebSettings::globalSettings()->setDefaultTextEncoding
	(ui.encodingCombinationBox->currentText().toLower());
      QWebSettings::globalSettings()->setAttribute
	(QWebSettings::JavaEnabled,
	 ui.javaCheckBox->isChecked());
      QWebSettings::globalSettings()->setAttribute
	(QWebSettings::JavascriptEnabled,
	 ui.javascriptGroupBox->isChecked());
      QWebSettings::globalSettings()->setAttribute
	(QWebSettings::JavascriptCanOpenWindows,
	 ui.jsAllowNewWindowsCheckBox->isChecked() &&
	 ui.javascriptGroupBox->isChecked());
      QWebSettings::globalSettings()->setAttribute
	(QWebSettings::AutoLoadImages,
	 ui.automaticallyLoadImagesCheckBox->isChecked());
      QWebSettings::globalSettings()->setAttribute
	(QWebSettings::PluginsEnabled,
	 ui.webPluginsCheckBox->isChecked());
      QWebSettings::globalSettings()->setFontFamily
	(QWebSettings::StandardFont,
	 ui.defaultFontSizeCombinationBox->currentText());
      QWebSettings::globalSettings()->setFontSize
	(QWebSettings::DefaultFontSize,
	 ui.defaultFontSizeCombinationBox->currentText().toInt());
      QWebSettings::globalSettings()->setFontFamily
	(QWebSettings::CursiveFont,
	 ui.cursiveFontCombinationBox->currentFont().family());
      QWebSettings::globalSettings()->setFontFamily
	(QWebSettings::FantasyFont,
	 ui.fantasyFontCombinationBox->currentFont().family());
      QWebSettings::globalSettings()->setFontFamily
	(QWebSettings::FixedFont,
	 ui.fixedFontCombinationBox->currentFont().family());
      QWebSettings::globalSettings()->setFontSize
	(QWebSettings::DefaultFixedFontSize,
	 ui.fixedFontSizeCombinationBox->currentText().toInt());
      QWebSettings::globalSettings()->setFontFamily
	(QWebSettings::SansSerifFont,
	 ui.sansSerifFontCombinationBox->currentFont().family());
      QWebSettings::globalSettings()->setFontFamily
	(QWebSettings::SerifFont,
	 ui.serifFontCombinationBox->currentFont().family());
      QWebSettings::globalSettings()->setFontSize
	(QWebSettings::MinimumFontSize,
	 ui.minimumFontSizeCombinationBox->currentText().toInt());
#if QT_VERSION >= 0x040700
      QWebSettings::globalSettings()->setAttribute
	(QWebSettings::SpatialNavigationEnabled,
	 ui.spatialNavigationCheckBox->isChecked());
      QWebSettings::globalSettings()->setAttribute
	(QWebSettings::XSSAuditingEnabled,
	 ui.xssAuditingCheckBox->isChecked());
#endif

      /*
      ** Populate the dooble::s_settings container.
      */

      for(int i = 0; i < settings.allKeys().size(); i++)
	dooble::s_settings[settings.allKeys().at(i)] =
	  settings.value(settings.allKeys().at(i));

      for(int i = 0; i < ui.iconSetComboBox->count(); i++)
	{
	  settings.setValue(QString("settingsWindow/iconSet%1").arg(i + 1),
			    ui.iconSetComboBox->itemData(i));
	  dooble::s_settings[QString("settingsWindow/iconSet%1").arg(i + 1)] =
	    ui.iconSetComboBox->itemData(i);
	}

      /*
      ** These signals should have been combined.
      */

      emit showTabBar(ui.alwaysShowTabBarCheckBox->isChecked());
      emit showIpAddress(ui.displayIpAddressCheckBox->isChecked());
      emit fontChanged(QApplication::font());
      emit iconsChanged();
      emit cookieTimerChanged();
      emit textSizeMultiplierChanged(ui.textSizeMultiplierSpinBox->value());
      slotSetIcons();

#ifndef Q_WS_MAC
      /*
      ** Ensure that the window is properly sized if the font has been
      ** changed.
      */

      resize(sizeHint());
#endif
    }

  close();
}

void dsettings::keyPressEvent(QKeyEvent *event)
{
  if(event && event->key() == Qt::Key_Escape)
    close();

  QMainWindow::keyPressEvent(event);
}

void dsettings::slotSelectIconCfgFile(void)
{
  QFileDialog fileDialog(this);

#ifdef Q_WS_MAC
  fileDialog.setAttribute(Qt::WA_MacMetalStyle, true);
#endif
  fileDialog.setWindowTitle(tr("Dooble Web Browser: Theme "
			       "Selection"));

  if(dooble::s_settings.contains("settingsWindow/themesPath"))
    {
      if(QFile::exists(dooble::s_settings.value("settingsWindow/themesPath").
		       toString()))
	fileDialog.setDirectory
	  (dooble::s_settings.value("settingsWindow/themesPath").toString());
      else
	fileDialog.setDirectory(QDir::currentPath());
    }
  else
    fileDialog.setDirectory(QDir::currentPath());

  fileDialog.setFileMode(QFileDialog::ExistingFile);
  fileDialog.setLabelText(QFileDialog::Accept, tr("&Select"));
  fileDialog.setNameFilter(tr("Theme Configuration File (*.cfg)"));

  if(fileDialog.exec() == QDialog::Accepted)
    {
      QStringList list(fileDialog.selectedFiles());

      if(!list.isEmpty())
	{
	  QDir dir(fileDialog.directory());
	  QSettings settings;

	  if(dir.cdUp())
	    {
	      settings.setValue("settingsWindow/themesPath",
				dir.absolutePath());
	      dooble::s_settings["settingsWindow/themesPath"] =
		dir.absolutePath();
	    }
	  else
	    {
	      settings.setValue("settingsWindow/themesPath",
				QDir::homePath());
	      dooble::s_settings["settingsWindow/themesPath"] =
		QDir::homePath();
	    }

	  bool itemInserted = false;
	  QSettings cfgSettings(list[0],
				QSettings::IniFormat);

	  cfgSettings.beginGroup("Description");
	  ui.iconSetComboBox->blockSignals(true);

	  for(int i = 0; i < ui.iconSetComboBox->count(); i++)
	    if(cfgSettings.value("Name").toString() <
	       ui.iconSetComboBox->itemText(i))
	      {
		itemInserted = true;
		ui.iconSetComboBox->insertItem
		  (i, cfgSettings.value("Name").toString(),
		   list[0]);
		ui.iconSetComboBox->setCurrentIndex(i);
		break;
	      }
	    else if(cfgSettings.value("Name").toString() ==
		    ui.iconSetComboBox->itemText(i))
	      {
		itemInserted = true;
		ui.iconSetComboBox->setCurrentIndex(i);
		break;
	      }

	  if(!itemInserted)
	    {
	      ui.iconSetComboBox->addItem
		(cfgSettings.value("Name").toString(),
		 list[0]);
	      ui.iconSetComboBox->setCurrentIndex
		(ui.iconSetComboBox->count() - 1);
	    }

	  ui.iconSetComboBox->blockSignals(false);
	  list.clear();
	  slotIconsPreview();
	}
    }
}

void dsettings::slotChooseMyRetrievedFilesDirectory(void)
{
  QFileDialog fileDialog(this);

#ifdef Q_WS_MAC
  fileDialog.setAttribute(Qt::WA_MacMetalStyle, true);
#endif
  fileDialog.setWindowTitle
    (tr("Dooble Web Browser: My Retrieved Files Directory Selection"));
  fileDialog.setFileMode(QFileDialog::Directory);
  fileDialog.setDirectory(QDir::homePath());
  fileDialog.setLabelText(QFileDialog::Accept, tr("&Open"));
  fileDialog.setAcceptMode(QFileDialog::AcceptOpen);

  if(fileDialog.exec() == QDialog::Accepted)
    {
      QStringList list(fileDialog.selectedFiles());

      if(!list.isEmpty())
	{
	  ui.myRetrievedFilesLineEdit->setText(list[0]);
	  list.clear();
	}
    }
}

void dsettings::slotAddBlockUrl(void)
{
}

void dsettings::updateFontWidgets(const QString &fontName,
				  QComboBox *fontSizeWidget)
{
  if(fontName.isEmpty() || !fontSizeWidget)
    return;

  fontSizeWidget->clear();

  QList<int> fontSizes;
  QFontDatabase fontDatabase;

  fontSizes = fontDatabase.pointSizes(fontName);

  for(int i = 0; i < fontSizes.size(); i++)
    fontSizeWidget->addItem(QString::number(fontSizes.at(i)));
}

void dsettings::slotFontChanged(const QString &text)
{
  if(text.isEmpty())
    return;

  updateFontWidgets(text, ui.fontSizeComboBox);
}

void dsettings::slotWebFontChanged(const QString &text)
{
  if(text.isEmpty())
    return;

  QComboBox *comboBox = qobject_cast<QComboBox *> (sender());

  if(!comboBox)
    return;

  if(comboBox == ui.defaultFontCombinationBox)
    updateFontWidgets(text, ui.defaultFontSizeCombinationBox);
  else if(comboBox == ui.fixedFontCombinationBox)
    updateFontWidgets(text, ui.fixedFontSizeCombinationBox);
}

void dsettings::slotSetIcons(void)
{
  QSettings settings
    (dooble::s_settings.value("iconSet").toString(),
     QSettings::IniFormat);

  foreach(QPushButton *button,
	  ui.buttonBox->findChildren<QPushButton *>())
    if(ui.buttonBox->buttonRole(button) == QDialogButtonBox::AcceptRole)
      button->setIcon
	(QIcon(settings.value("okButtonIcon").toString()));
    else
      button->setIcon
	(QIcon(settings.value("cancelButtonIcon").toString()));

  ui.displayButton->setIcon
    (QIcon(settings.value("settingsWindow/displayButtonIcon").toString()));
  ui.historyButton->setIcon
    (QIcon(settings.value("settingsWindow/historyButtonIcon").toString()));
  ui.homeButton->setIcon
    (QIcon(settings.value("settingsWindow/homeButtonIcon").toString()));
  ui.proxyButton->setIcon
    (QIcon(settings.value("settingsWindow/proxyButtonIcon").toString()));
  ui.securityButton->setIcon
    (QIcon(settings.value("settingsWindow/securityButtonIcon").toString()));
  ui.tabsButton->setIcon
    (QIcon(settings.value("settingsWindow/tabsButtonIcon").toString()));
  ui.webkitButton->setIcon
    (QIcon(settings.value("settingsWindow/webkitButtonIcon").toString()));
  ui.windowsButton->setIcon
    (QIcon(settings.value("settingsWindow/windowsButtonIcon").toString()));
  ui.safeButton->setIcon
    (QIcon(settings.value("settingsWindow/safeButtonIcon").toString()));

  int h = 0;
  int w = 0;

  foreach(QToolButton *button,
	  ui.tabScrollArea->findChildren<QToolButton *> ())
    {
      if(button->sizeHint().width() > w)
	w = button->sizeHint().width();

      if(button->sizeHint().height() > h)
	h = button->sizeHint().height();
    }

  foreach(QToolButton *button,
	  ui.tabScrollArea->findChildren<QToolButton *> ())
    button->setMinimumSize(w, h);

  ui.tabScrollArea->setMinimumHeight(h + 25);
}

void dsettings::slotChangePage(bool checked)
{
  QToolButton *button = qobject_cast<QToolButton *> (sender());

  if(!button)
    return;
  else if(!checked)
    {
      button->setChecked(true);
      button->setFocus();
      return;
    }

  foreach(QToolButton *btn, ui.tabScrollArea->findChildren<QToolButton *> ())
    if(btn != button)
      btn->setChecked(!checked);
    else
      btn->setChecked(checked);

  button->setFocus();
  ui.stackedWidget->setCurrentIndex(button->property("page").toInt());
}

void dsettings::slotCookieTimerTimeChanged(int index)
{
  int value = ui.cookieTimerSpinBox->value();

  if(index == 0)
    {
      ui.cookieTimerSpinBox->setMinimum(1);
      ui.cookieTimerSpinBox->setMaximum(60);
    }
  else
    {
      ui.cookieTimerSpinBox->setMinimum(1);
      ui.cookieTimerSpinBox->setMaximum(3600);
    }

  ui.cookieTimerSpinBox->setValue(value);
}

void dsettings::closeEvent(QCloseEvent *event)
{
  ui.pass1LineEdit->clear();
  ui.pass2LineEdit->clear();

  if(!m_previousIconSet.isEmpty())
    {
      dooble::s_settings["iconSet"] = m_previousIconSet;
      emit iconsChanged();
    }

  m_previousIconSet.clear();
  QMainWindow::closeEvent(event);
}

void dsettings::slotEnablePassphrase(void)
{
  ui.passphraseGroupBox->setEnabled(true);
}

void dsettings::slotGroupBoxClicked(bool checked)
{
  QGroupBox *groupBox = qobject_cast<QGroupBox *> (sender());

  if(groupBox == ui.cookieGroupBox)
    {
      ui.cookieTimerSpinBox->setEnabled
	(checked && ui.cookieTimerCheckBox->isChecked());
      ui.cookieTimerComboBox->setEnabled
	(checked && ui.cookieTimerCheckBox->isChecked());
    }
}

void dsettings::slotPassphraseWasAuthenticated(const bool state)
{
  /*
  ** This slot is connected to dooble's passphraseWasAuthenticated()
  ** signal.
  */

  ui.passphraseGroupBox->setEnabled(false);
  ui.changePassphrasePushButton->setEnabled(state);
}

void dsettings::slotIconsPreview(void)
{
  if(m_previousIconSet.isEmpty())
    m_previousIconSet = dooble::s_settings["iconSet"].toString();

  dooble::s_settings["iconSet"] =
    ui.iconSetComboBox->itemData(ui.iconSetComboBox->currentIndex()).
    toString();
  emit iconsChanged();
}

dooble *dsettings::parentDooble(void) const
{
  return m_parentDooble;
}
