/****************************************************************************
** Dooble - The Secure Internet Web Browser
**
** Copyright (c) 2008, 2009, 2010, 2011 Alexis Megas,
** Gunther van Dooble, and the Dooble Team.
** All rights reserved.
**
** License: GPL2 only:
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; version 2 of the License only.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
** or see here: http://www.gnu.org/licenses/gpl.html
**
** For the WebKit library, please see: http://webkit.org.
**
** THE CODE IS PROVIDED BY THE AUTHORS ''AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
** ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
** IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS APPLICATION, EVEN IF ADVISED
** OF THE POSSIBILITY OF SUCH DAMAGE.
**
** Please report all praise, requests, bugs, and problems to the project
** team and administrators: http://sf.net/projects/dooble.
**
** You can find us listed at our project page. New team members are welcome.
** The name of the authors should not be used to endorse or promote products
** derived from Dooble without specific prior written permission.
** If you use this code for other projects, please let us know.
**
** Web sites:
**   http://sf.net/projects/dooble
**   http://dooble.sf.net
****************************************************************************/

#include <QMenu>
#include <QMovie>
#include <QTabBar>
#include <QSettings>
#include <QDropEvent>
#include <QToolButton>

#include "dooble.h"
#include "dtabwidget.h"

dtabbar::dtabbar(QWidget *parent):QTabBar(parent)
{
  setExpanding(false);
  setAcceptDrops(true);
  setUsesScrollButtons(true);
}

void dtabbar::dropEvent(QDropEvent *event)
{
  if(event && event->source() &&
     event->source()->objectName() == "dooble_history_sidebar" &&
     event->mimeData() && event->mimeData()->hasUrls())
    {
      QList<QUrl> list(event->mimeData()->urls());

      event->accept();
      emit urlsReceivedViaDrop(list);
      list.clear();
      return;
    }

  QTabBar::dropEvent(event);
}

void dtabbar::dragMoveEvent(QDragMoveEvent *event)
{
  if(event && event->source() &&
     event->source()->objectName() == "dooble_history_sidebar" &&
     event->mimeData() && event->mimeData()->hasUrls())
    {
      event->accept();
      return;
    }

  QTabBar::dragMoveEvent(event);
}

void dtabbar::dragEnterEvent(QDragEnterEvent *event)
{
  if(event && event->source() &&
     event->source()->objectName() == "dooble_history_sidebar" &&
     event->mimeData() && event->mimeData()->hasUrls())
    {
      event->accept();
      return;
    }

  QTabBar::dragEnterEvent(event);
}

QSize dtabbar::sizeHint(void) const
{
  return QTabBar::sizeHint();
}

dtabwidget::dtabwidget(QWidget *parent):QTabWidget(parent)
{
  if(tabBar())
    tabBar()->deleteLater();

  setTabBar(new dtabbar(this));
  setAcceptDrops(true);
#ifdef Q_WS_MAC
  setDocumentMode(true);
#endif
  setElideMode(Qt::ElideRight);
  selectedTabIndex = -1;
  setStyleSheet("QTabWidget::tab-bar {"
		"alignment: left; "
#ifdef Q_WS_X11
		"}"
		"QTabWidget::pane {"
		"border-top: 1px solid lightgray; "
		"border-bottom: 1px solid lightgray; "
		"position: absolute; "
		"top: 0em; "
#endif
		"}");
  tabBar()->setStyleSheet
    ("QTabBar::tear {"
     "image: none; "
     "}");
#ifndef Q_WS_MAC
  tabBar()->setStyleSheet
    (tabBar()->styleSheet() +
     "QTabBar::tab:selected {"
     "background-color:"
     "qlineargradient(spread:reflect, x1:0.023,"
     "y1:0.988636, x2:0.028, y2:0.011,"
     "stop:0 rgba(218, 232, 255, 255),"
     "stop:1 rgba(170, 85, 255, 255))"
     "}");
#endif
  tabBar()->setStyleSheet(tabBar()->styleSheet() +
			  "QTabBar::tab {"
#ifndef Q_WS_WIN
			  "max-width: 30ex; "
			  "min-width: 30ex; "
#endif
#ifdef Q_WS_MAC
			  // Nothing?
#elif defined Q_WS_WIN
			  "max-width: 50ex; "
			  "min-width: 50ex; "
#else
			  "min-height: 3ex; "
			  "border: 1px solid gray; "
#endif
			  "}");
  tabBar()->setContextMenuPolicy(Qt::CustomContextMenu);
  tabBar()->setSelectionBehaviorOnRemove(QTabBar::SelectPreviousTab);
  connect(tabBar(), SIGNAL(customContextMenuRequested(const QPoint &)),
	  this,
	  SLOT(slotShowContextMenu(const QPoint &)));
  connect(tabBar(), SIGNAL(tabMoved(int, int)),
	  this,
	  SIGNAL(tabMoved(int, int)));
  connect(tabBar(), SIGNAL(urlsReceivedViaDrop(const QList<QUrl> &)),
	  this, SIGNAL(urlsReceivedViaDrop(const QList<QUrl> &)));
  setMovable(true);
  setTabsClosable(true);
  connect(this,
	  SIGNAL(tabCloseRequested(int)),
	  this,
	  SLOT(slotCloseTab(int)));
  addTabButton = new QToolButton();
#ifdef Q_WS_X11
  addTabButton->setStyleSheet("QToolButton {"
			      "border: none; "
			      "min-height: 3ex; "
			      "}");
#endif
  addTabButton->setToolTip(tr("Add New Page"));
  connect(addTabButton,
	  SIGNAL(clicked(void)),
	  this,
	  SLOT(slotCreateTab(void)));
  setCornerWidget(addTabButton, Qt::TopRightCorner);
#ifdef Q_WS_MAC
  /*
  ** The directional arrows (left and right) have
  ** transparent backgrounds, causing the foreground
  ** of the tab beneath the buttons to seep into the
  ** foreground.
  */

  foreach(QToolButton *toolButton, tabBar()->findChildren
	  <QToolButton *> ())
    toolButton->setStyleSheet
    ("QToolButton {border: none; background-color: white}"
     "QToolButton::menu-button {border: none;}");
#endif
  slotSetIcons();
}

dtabwidget::~dtabwidget()
{
}

void dtabwidget::slotShowContextMenu(const QPoint &point)
{
  selectedTabIndex = tabBar()->tabAt(point);

  if(selectedTabIndex > -1)
    {
      if(qobject_cast<dview *> (widget(selectedTabIndex)))
	{
	  QMenu menu(this);
	  QAction *action = 0;

	  action = menu.addAction(tr("Close Ta&b"),
				  this, SLOT(slotCloseTab(void)));
	  action->setEnabled(count() != 1);
	  action = menu.addAction(tr("Close &Other Tabs"),
				  this, SLOT(slotCloseOtherTabs(void)));
	  action->setEnabled(count() > 1);
	  menu.addSeparator();
	  menu.addAction(tr("New &Tab"),
			 this, SLOT(slotCreateTab(void)));
	  action = menu.addAction(tr("Open in &New Window"),
				  this, SLOT(slotOpenInNewWindow(void)));
	  action->setEnabled(count() > 1);
	  menu.exec(tabBar()->mapToGlobal(point));
	}
      else
	{
	  QMenu menu(this);
	  QAction *action = 0;

	  action = menu.addAction(tr("Close Ta&b"),
				  this, SLOT(slotCloseTab(void)));
	  action->setEnabled(count() != 1);
	  action = menu.addAction(tr("Close &Other Tabs"),
				  this, SLOT(slotCloseOtherTabs(void)));
	  action->setEnabled(count() > 1);
	  menu.addSeparator();
	  menu.addAction(tr("New &Tab"),
			 this, SLOT(slotCreateTab(void)));
	  menu.exec(tabBar()->mapToGlobal(point));
	}
    }
}

void dtabwidget::slotCloseTab(int index)
{
  if(index >= 0 && index < count())
    emit closeTab(index);
}

void dtabwidget::slotCloseTab(void)
{
  emit closeTab(selectedTabIndex);
  selectedTabIndex = -1;
}

void dtabwidget::slotCreateTab(void)
{
  emit createTab();
}

void dtabwidget::slotOpenInNewWindow(void)
{
  emit openInNewWindow(selectedTabIndex);
  selectedTabIndex = -1;
}

void dtabwidget::tabRemoved(int index)
{
  Q_UNUSED(index);
}

void dtabwidget::setBarVisible(const bool state)
{
  tabBar()->setVisible(state);
  addTabButton->setVisible(state);
}

void dtabwidget::slotSetIcons(void)
{
  QSettings settings
    (dooble::s_settings.value("iconSet").toString(), QSettings::IniFormat);

  settings.beginGroup("mainWindow");
  addTabButton->setIcon(QIcon(settings.value("addTab").toString()));
}

void dtabwidget::mouseDoubleClickEvent(QMouseEvent *event)
{
  if(event && event->button() == Qt::LeftButton)
    {
      if(dooble::s_settings.value("settingsWindow/addTabWithDoubleClick",
				  true).toBool())
	emit createTab();
    }

  QTabWidget::mouseDoubleClickEvent(event);
}

void dtabwidget::mousePressEvent(QMouseEvent *event)
{
  if(event && event->button() == Qt::MidButton)
    {
      if(!dooble::s_settings.value("settingsWindow/closeViaMiddle",
				   true).toBool())
	return;

      int index = tabBar()->tabAt(event->pos());

      if(index != -1)
	emit closeTab(index);
    }

  QTabWidget::mousePressEvent(event);
}

void dtabwidget::slotCloseOtherTabs(void)
{
  int index = selectedTabIndex;

  if(index == -1)
    return;

  /*
  ** The slot that's connected to the closeTab() signal issues
  ** deleteLater() calls. Timing should not be a concern.
  */

  for(int i = count() - 1; i > index; i--)
    emit closeTab(i);

  for(int i = index - 1; i >= 0; i--)
    emit closeTab(i);

  selectedTabIndex = -1;
}

void dtabwidget::setTabButton(int index)
{
  QTabBar::ButtonPosition side = (QTabBar::ButtonPosition) style()->styleHint
    (QStyle::SH_TabBar_CloseButtonPosition, 0, tabBar());

  side = (side == QTabBar::LeftSide) ? QTabBar::RightSide : QTabBar::LeftSide;

  QLabel *label = qobject_cast<QLabel *> (tabBar()->tabButton(index, side));

  if(!label)
    {
      label = new QLabel(tabBar());

      QPixmap pixmap(16, 16);

      pixmap.fill(tabBar()->backgroundRole());
      label->setPixmap(pixmap);
    }

  tabBar()->setTabButton(index, side, 0);
  tabBar()->setTabButton(index, side, label);
}

void dtabwidget::animateIndex(const int index, const bool state,
			      const QIcon &icon)
{
  QTabBar::ButtonPosition side = (QTabBar::ButtonPosition) style()->styleHint
    (QStyle::SH_TabBar_CloseButtonPosition, 0, tabBar());

  side = (side == QTabBar::LeftSide) ? QTabBar::RightSide : QTabBar::LeftSide;

  QLabel *label = qobject_cast<QLabel*>
    (tabBar()->tabButton(index, side));

  if(label)
    {
      if(state)
	{
	  if(!label->movie())
	    {
	      QMovie *movie = new QMovie
		("Tab/Default/indicator.gif", QByteArray(), label);

	      movie->setScaledSize(QSize(16, 16));
	      label->setMovie(movie);
	      movie->start();
	    }
	}
      else
	{
	  if(label->movie())
	    {
	      label->movie()->stop();
	      label->movie()->deleteLater();
	      label->setMovie(0);
	    }

	  label->setPixmap(icon.pixmap(16, 16));
	}
    }
}

void dtabwidget::dropEvent(QDropEvent *event)
{
  if(event && event->source() &&
     event->source()->objectName() == "dooble_history_sidebar" &&
     event->mimeData() && event->mimeData()->hasUrls())
    {
      int index = tabBar()->tabAt(event->pos());
      QList<QUrl> list(event->mimeData()->urls());

      if(index == -1)
	{
	  while(!list.isEmpty())
	    emit openLinkInNewTab(list.takeFirst());
	}
      else
	emit urlsReceivedViaDrop(list);

      return;
    }

  QTabWidget::dropEvent(event);
}

void dtabwidget::dragMoveEvent(QDragMoveEvent *event)
{
  if(event && event->source() &&
     event->source()->objectName() == "dooble_history_sidebar" &&
     event->mimeData() && event->mimeData()->hasUrls())
    {
      event->accept();
      return;
    }

  QTabWidget::dragMoveEvent(event);
}

void dtabwidget::dragEnterEvent(QDragEnterEvent *event)
{
  if(event && event->source() &&
     event->source()->objectName() == "dooble_history_sidebar" &&
     event->mimeData() && event->mimeData()->hasUrls())
    {
      event->accept();
      return;
    }

  QTabWidget::dragEnterEvent(event);
}

void dtabwidget::slotIconChange (QWidget *tab, const QIcon & icon)
{
  int tabno = indexOf (tab);
  if (tabno >= 0) 
    {
      setTabIcon (tabno, icon);
    }
}

void dtabwidget::slotTitleChange (QWidget *tab, const QString & title)
{
  int tabno = indexOf (tab);
  if (tabno >= 0) 
    {
      setTabText (tabno, title);
    }
}
