/****************************************************************************
** Dooble - The Secure Internet Web Browser
**
** Copyright (c) 2008, 2009, 2010, 2011 Alexis Megas,
** Gunther van Dooble, and the Dooble Team.
** All rights reserved.
**
** License: GPL2 only:
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; version 2 of the License only.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
** or see here: http://www.gnu.org/licenses/gpl.html
**
** For the WebKit library, please see: http://webkit.org.
**
** THE CODE IS PROVIDED BY THE AUTHORS ''AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
** ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
** IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS APPLICATION, EVEN IF ADVISED
** OF THE POSSIBILITY OF SUCH DAMAGE.
**
** Please report all praise, requests, bugs, and problems to the project
** team and administrators: http://sf.net/projects/dooble.
**
** You can find us listed at our project page. New team members are welcome.
** The name of the authors should not be used to endorse or promote products
** derived from Dooble without specific prior written permission.
** If you use this code for other projects, please let us know.
**
** Web sites:
**   http://sf.net/projects/dooble
**   http://dooble.sf.net
****************************************************************************/

#include <QUrl>
#include <QMenu>
#include <QStack>
#include <QStyle>
#include <QBuffer>
#include <QWebView>
#include <QFileInfo>
#include <QBoxLayout>
#include <QClipboard>
#include <QPushButton>
#include <QWebHistory>
#ifdef DOOBLE_X11_FLASH_BYPASS
#include <QResizeEvent>
#endif
#include <QNetworkProxy>
#include <QContextMenuEvent>
#include <QFileIconProvider>
#include <QWebHitTestResult>

#include "dmisc.h"
#include "dview.h"
#include "dooble.h"
#include "dwebpage.h"
#include "dsettings.h"
#include "durlwidget.h"

dview::dview(QWidget *parent, dwebhistory *history):QStackedWidget(parent)
{
  m_action = 0;
#ifndef DOOBLE_X11_FLASH_BYPASS
  webView = new dwebview(this);
  webView->setRenderHints(QPainter::Antialiasing |
			  QPainter::TextAntialiasing |
			  QPainter::SmoothPixmapTransform |
			  QPainter::HighQualityAntialiasing);
#else
  m_view = new QGraphicsView(this);
  m_view->setRenderHints(QPainter::Antialiasing |
			 QPainter::TextAntialiasing |
			 QPainter::SmoothPixmapTransform |
			 QPainter::HighQualityAntialiasing);
  m_view->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
  m_view->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
  m_scene = new QGraphicsScene(m_view);
  m_view->setScene(m_scene);
  m_view->setSceneRect(QRectF());
  m_view->setFrameShape(QFrame::NoFrame);
  webView = new dwebview(0);
  m_scene->addItem(webView);
  m_view->setViewportUpdateMode(QGraphicsView::SmartViewportUpdate);
#endif
  ftpBrowser = 0;
  fileManager = 0;
  directoryView = 0;
  m_history = new dwebhistory(history);
  webView->setPage(new dwebpage(webView));

  /*
  ** Please set the text size multiplier after creating the dwebpage
  ** object.
  */

#ifndef DOOBLE_X11_FLASH_BYPASS
  webView->setTextSizeMultiplier
    (dooble::s_settings.value("settingsWindow/textSizeMultiplier",
			      1.00).toReal());
#endif
  webView->page()->setLinkDelegationPolicy(QWebPage::DontDelegateLinks);
  webView->page()->networkAccessManager()->setCookieJar(dooble::s_cookies);
  dooble::s_cookies->setParent(0);
#ifndef DOOBLE_X11_FLASH_BYPASS
  addWidget(webView);
#else
  addWidget(m_view);
#endif
#ifndef DOOBLE_X11_FLASH_BYPASS
  setCurrentWidget(webView);
#else
  setCurrentWidget(m_view);
#endif
  m_pageLoaded = false;
  m_percentLoaded = 0;
  selectedUrl = "";
  selectedImageUrl = "";
#ifndef DOOBLE_X11_FLASH_BYPASS
  webView->setContextMenuPolicy(Qt::CustomContextMenu);
#else
  m_view->setContextMenuPolicy(Qt::CustomContextMenu);
#endif
  connect(webView, SIGNAL(urlChanged(const QUrl &)), this,
	  SLOT(slotUrlChanged(const QUrl &)));
  connect(webView, SIGNAL(iconChanged(void)), this,
	  SLOT(slotIconChanged(void)));
  connect(webView, SIGNAL(titleChanged(const QString &)), this,
	  SLOT(slotTitleChanged(const QString &)));
  connect(webView, SIGNAL(loadFinished(bool)), this,
	  SLOT(slotLoadFinished(bool)));
  connect(webView, SIGNAL(loadStarted(void)), this,
	  SLOT(slotLoadStarted(void)));
  connect(webView, SIGNAL(loadProgress(int)), this,
	  SLOT(slotLoadProgress(int)));
#ifndef DOOBLE_X11_FLASH_BYPASS
  connect(webView, SIGNAL(customContextMenuRequested(const QPoint &)),
	  this, SLOT(slotCustomContextMenuRequested(const QPoint &)));
#else
  connect(m_view, SIGNAL(customContextMenuRequested(const QPoint &)),
	  this, SLOT(slotCustomContextMenuRequested(const QPoint &)));
#endif
  connect(webView, SIGNAL(linkClicked(const QUrl &)),
	  this, SLOT(slotLinkClicked(const QUrl &)));
  connect(dooble::s_settingsWindow,
	  SIGNAL(textSizeMultiplierChanged(const qreal)),
	  this,
	  SLOT(slotSetTextSizeMultiplier(const qreal)));
  connect(webView->page()->networkAccessManager(),
	  SIGNAL(sslErrors(QNetworkReply *, const QList<QSslError> &)),
	  this,
	  SLOT(slotSslErrors(QNetworkReply *, const QList<QSslError> &)));
  connect(webView->page(),
	  SIGNAL(unsupportedContent(QNetworkReply *)),
	  this,
	  SLOT(slotHandleUnsupportedContent(QNetworkReply *)));
  connect(webView->page(),
	  SIGNAL(selectionChanged(void)),
	  this,
	  SLOT(slotSelectionChanged(void)));
  connect(webView,
	  SIGNAL(urlsReceivedViaDrop(const QList<QUrl> &)),
	  this,
	  SIGNAL(urlsReceivedViaDrop(const QList<QUrl> &)));
}

dview::~dview()
{
  if(m_history)
    {
      delete m_history;
      m_history = 0;
    }

  if(m_action)
    m_action->deleteLater();
}

void dview::slotLoadFinished(bool ok)
{
  m_pageLoaded = true;

  dwebhistoryitem item;

  item.setUrl(url());
  item.setIcon(icon());
  item.setTitle(title());
  m_history->add(item);

  if(webView == sender())
    {
      webView->update();

#ifdef DOOBLE_X11_FLASH_BYPASS
      if(isVisible())
	{
	  m_view->setSceneRect(0, 0,
			       webView->size().width(),
			       webView->size().height());

	  /*
	  ** I have yet to arrive at an example that requires
	  ** fitInView() to be used. However, Qt is quirky.
	  */

	  m_view->fitInView(webView);
	}
#endif
    }
  else if(qobject_cast<dfilemanager *> (sender()))
    {
      initializeDirectoryView();

      if(ok)
	directoryView->setHtml(fileManager->html());
    }
  else if(qobject_cast<dftpbrowser *> (sender()))
    {
      initializeDirectoryView();

      if(ok)
	directoryView->setHtml(ftpBrowser->html());
    }

  emit loadFinished(ok);
}

void dview::slotLoadStarted(void)
{
  m_pageLoaded = false;
  emit loadStarted();
}

void dview::slotLoadProgress(int progress)
{
  if(progress >= 0)
    m_percentLoaded = progress;

  emit loadProgress(progress);
}

int dview::progress(void) const
{
  return m_percentLoaded;
}

bool dview::isLoaded(void) const
{
  return m_pageLoaded;
}

void dview::slotCustomContextMenuRequested(const QPoint &pos)
{
  QMenu menu(this);
  QWebHitTestResult result = webView->page()->currentFrame()->
    hitTestContent(pos);

  if(!result.linkUrl().isEmpty())
    {
      selectedUrl = result.linkUrl();

      if(selectedUrl.scheme().toLower() == "mailto")
	menu.addAction(tr("Copy &E-Mail Address"),
		       this, SLOT(slotCopyLinkLocation(void)));
      else
	{
	  menu.addAction(tr("Copy &Link Location"),
			 this, SLOT(slotCopyLinkLocation(void)));
	  menu.addAction(tr("Copy &Selected Text"),
			 this, SLOT(slotCopySelectedText(void)));
	  menu.addSeparator();
	  menu.addAction(tr("Open Link in New &Tab"),
			 this, SLOT(slotOpenLinkInNewTab(void)));
	  menu.addAction(tr("Open Link in &New Window"),
			 this, SLOT(slotOpenLinkInNewWindow(void)));
	  menu.addSeparator();
	  menu.addAction(tr("&Save Link"),
			 this, SLOT(slotSaveLink(void)));
	}
    }

  if(!result.imageUrl().isEmpty())
    {
      selectedImageUrl = result.imageUrl();

      if(!result.linkUrl().isEmpty())
	menu.addSeparator();

      menu.addSeparator();
      menu.addAction(tr("Copy &Image Location"),
		     this, SLOT(slotCopyImageLocation(void)));
      menu.addAction(tr("Sa&ve Image"),
		     this, SLOT(slotSaveImage(void)));
      menu.addAction(tr("View I&mage"),
		     this, SLOT(slotViewImage(void)));
      menu.addAction(tr("View Im&age in New Tab"),
		     this, SLOT(slotOpenImageInNewTab(void)));
      menu.addAction(tr("View Ima&ge in New Window"),
		     this, SLOT(slotOpenImageInNewWindow(void)));
    }

  if(menu.isEmpty())
    {
      menu.addAction(tr("&Copy"),
		     this,
		     SLOT(slotCopySelectedText(void)));
      menu.addAction(tr("&Paste"),
		     this,
		     SLOT(slotPaste(void)));
    }

  if(!menu.isEmpty())
    menu.addSeparator();

  QAction *action = 0;

  action = menu.addAction(tr("&Back"),
			  this,
			  SIGNAL(goBack(void)));
  action->setEnabled(canGoBack());
  action = menu.addAction(tr("&Forward"),
			  this,
			  SIGNAL(goForward(void)));
  action->setEnabled(canGoForward());
  menu.addAction(tr("&Reload"),
		 this,
		 SIGNAL(goReload(void)));
  action = menu.addAction(tr("&Stop"),
			  this,
			  SLOT(slotStop(void)));
  action->setEnabled(!m_pageLoaded);
  menu.addSeparator();

  if(webView->page()->mainFrame() !=
     webView->page()->currentFrame())
    menu.addAction(tr("&View Frame Source"),
		   this,
		   SLOT(slotViewPageSource(void)));
  else
    menu.addAction(tr("&View Page Source"),
		   this,
		   SLOT(slotViewPageSource(void)));

  menu.exec(mapToGlobal(pos));
}

void dview::slotAdBlock(void)
{
  /*
  ** Soon.
  */
}

void dview::slotViewImage(void)
{
  emit viewImage(selectedImageUrl);
}

void dview::slotOpenLinkInNewTab(void)
{
  emit openLinkInNewTab(selectedUrl);
}

void dview::slotOpenLinkInNewTab(const QUrl &url)
{
  emit openLinkInNewTab(url);
}

void dview::slotOpenLinkInNewWindow(void)
{
  emit openLinkInNewWindow(selectedUrl);
}

void dview::slotOpenLinkInNewWindow(const QUrl &url)
{
  emit openLinkInNewWindow(url);
}

void dview::slotOpenImageInNewTab(void)
{
  emit openLinkInNewTab(selectedImageUrl);
}

void dview::slotOpenImageInNewWindow(void)
{
  emit openLinkInNewWindow(selectedImageUrl);
}

void dview::slotCopyLinkLocation(void)
{
  if(selectedUrl.scheme().toLower() == "mailto")
    {
      QString str("");

      str = selectedUrl.toString
	(QUrl::RemoveScheme |
	 QUrl::RemoveUserInfo |
	 QUrl::RemovePort |
	 QUrl::RemoveQuery |
	 QUrl::RemoveFragment |
	 QUrl::StripTrailingSlash);
      emit copyLink(QUrl(str));
    }
  else
    emit copyLink(selectedUrl);
}

void dview::slotCopyLinkLocation(const QUrl &url)
{
  emit copyLink(url);
}

void dview::slotCopyImageLocation(void)
{
  emit copyLink(selectedImageUrl);
}

void dview::slotSaveLink(void)
{
  emit saveUrl(selectedUrl);
}

void dview::slotSaveLink(const QUrl &url)
{
  emit saveUrl(url);
}

void dview::slotSaveImage(void)
{
  emit saveUrl(selectedImageUrl);
}

void dview::slotClearHistory(void)
{
  m_history->clear();
  webView->history()->clear();

  if(ftpBrowser)
    ftpBrowser->history()->clear();

  if(fileManager)
    fileManager->history()->clear();
}

void dview::load(const QUrl &url)
{
  if(url.isEmpty() || !url.isValid())
    {
      m_pageLoaded = true;
      emit loadFinished(!m_pageLoaded);
      return;
    }

  QWidget *widget = widgetForUrl(url);

  if(!widget)
    {
      m_pageLoaded = true;
      emit loadFinished(!m_pageLoaded);
      return;
    }

  stop();

#ifndef DOOBLE_X11_FLASH_BYPASS
  if(qobject_cast<dwebview *> (widget))
#else
  if(qobject_cast<QGraphicsView *> (widget))
#endif
    {
#ifndef DOOBLE_X11_FLASH_BYPASS
      setCurrentWidget(webView);
#else
      setCurrentWidget(m_view);
#endif
      webView->page()->networkAccessManager()->setProxy
	(dmisc::proxyByUrl(url));

      if(dooble::s_javaScriptExceptionsWindow->allowed(url.host()))
	/*
	** Exempt hosts must be prevented from executing JavaScript.
	*/

	webView->settings()->setAttribute(QWebSettings::JavascriptEnabled,
					  false);
      else
	webView->settings()->setAttribute
	  (QWebSettings::JavascriptEnabled,
	   dooble::s_settings.value("settingsWindow/javascriptEnabled", true).
	   toBool());

      webView->load(url);
      webView->setProperty("temporary_url", url);
    }
  else if(qobject_cast<dftpbrowser *> (widget))
    {
      initializeFtpBrowser();
      setCurrentWidget(ftpBrowser);
      ftpBrowser->load(url);
    }
  else if(qobject_cast<dfilemanager *> (widget))
    {
      initializeFileManager();
      setCurrentWidget(fileManager);
      fileManager->load(url);
    }
}

QIcon dview::icon(void) const
{
#ifndef DOOBLE_X11_FLASH_BYPASS
  if(qobject_cast<dwebview *> (currentWidget()))
#else
  if(qobject_cast<QGraphicsView *> (currentWidget()))
#endif
    {
      if(!webView->icon().isNull())
	return webView->icon();
    }

  return dmisc::iconForUrl(url());
}

void dview::slotCopySelectedText(void)
{
  webView->triggerPageAction(QWebPage::Copy);
}

dwebpage *dview::page(void) const
{
  return qobject_cast<dwebpage *> (webView->page());
}

QUrl dview::url(void) const
{
#ifndef DOOBLE_X11_FLASH_BYPASS
  if(qobject_cast<dwebview *> (currentWidget()))
#else
  if(qobject_cast<QGraphicsView *> (currentWidget()))
#endif
    {
      QUrl l_url(webView->property("temporary_url").toUrl());

      if(l_url.toString(QUrl::StripTrailingSlash) !=
	 webView->url().toString(QUrl::StripTrailingSlash))
	return l_url;
      else
	return webView->url();
    }
  else if(qobject_cast<dftpbrowser *> (currentWidget()))
    return ftpBrowser->url();
  else if(qobject_cast<dfilemanager *> (currentWidget()))
    return fileManager->url();

  return QUrl();
}

void dview::stop(void)
{
  /*
  ** The page has been loaded, even if partially.
  */

  m_pageLoaded = true;
  webView->stop();

  if(ftpBrowser)
    ftpBrowser->stop();
}

void dview::reload(void)
{
  stop();

#ifndef DOOBLE_X11_FLASH_BYPASS
  if(qobject_cast<dwebview *> (currentWidget()))
#else
  if(qobject_cast<QGraphicsView *> (currentWidget()))
#endif
    {
      QUrl l_url(webView->property("temporary_url").toUrl());

      if(l_url.toString(QUrl::StripTrailingSlash) !=
	 webView->url().toString(QUrl::StripTrailingSlash))
	load(l_url);
      else
	load(webView->url());
    }
  else if(qobject_cast<dftpbrowser *> (currentWidget()))
    ftpBrowser->reload();
  else if(qobject_cast<dfilemanager *> (currentWidget()))
    fileManager->reload();
}

void dview::back(void)
{
  QWidget *widget = 0;
  dwebhistoryitem item(m_history->back());

  if((widget = widgetForUrl(item.url())) == 0)
    {
      m_pageLoaded = true;
      emit loadFinished(!m_pageLoaded);
      return;
    }
  else if(item.url().isEmpty() ||
	  !item.url().isValid())
    {
      m_pageLoaded = true;
      emit loadFinished(!m_pageLoaded);
      return;
    }

  stop();

  QWidget *prevWidget = currentWidget();

#ifndef DOOBLE_X11_FLASH_BYPASS
  if(qobject_cast<dwebview *> (widget))
#else
  if(qobject_cast<QGraphicsView *> (widget))
#endif
    {
#ifndef DOOBLE_X11_FLASH_BYPASS
      setCurrentWidget(webView);
#else
      setCurrentWidget(m_view);
#endif

      bool doLoad = false;

      if(item.url().toString(QUrl::StripTrailingSlash) !=
	 webView->history()->backItem().url().
	 toString(QUrl::StripTrailingSlash))
	/*
	** QWebView's history's first back item may be the current url.
	*/

	doLoad = true;

      if(prevWidget == currentWidget())
	{
	  bool emitSignals = false;
	  bool url1HasFragment = false;
	  bool url2HasFragment = false;
	  QUrl url1(webView->url());
	  QUrl url2(webView->history()->backItem().url());

	  /*
	  ** If we're going from an URL that has a fragment
	  ** to an URL that is equal to the current URL without
	  ** the fragment, we need to emit signals. Similarly
	  ** for the reverse.
	  */

	  if((url1HasFragment = url1.hasFragment()))
	    url1.setFragment(QString());

	  if((url2HasFragment = url2.hasFragment()))
	    url2.setFragment(QString());

	  if((url1 == url2 && (url1HasFragment || url2HasFragment)) ||
	     url2.isEmpty())
	    emitSignals = true;

	  webView->back();

	  if(emitSignals)
	    {
	      m_pageLoaded = true;
	      emit loadFinished(m_pageLoaded);
	    }

	  if(url2HasFragment)
	    doLoad = true;
	}
      else if(!doLoad)
	load(item.url());

      if(doLoad)
	load(item.url());
    }
  else if(qobject_cast<dftpbrowser *> (widget))
    {
      setCurrentWidget(ftpBrowser);

      if(prevWidget == currentWidget())
	ftpBrowser->back();
      else
	load(item.url());
    }
  else if(qobject_cast<dfilemanager *> (widget))
    {
      setCurrentWidget(fileManager);

      if(prevWidget == currentWidget())
	fileManager->back();
      else
	load(item.url());
    }
}

void dview::forward(void)
{
  QWidget *widget = 0;
  dwebhistoryitem item(m_history->forward());

  if((widget = widgetForUrl(item.url())) == 0)
    {
      m_pageLoaded = true;
      emit loadFinished(!m_pageLoaded);
      return;
    }
  else if(item.url().isEmpty() ||
	  !item.url().isValid())
    {
      m_pageLoaded = true;
      emit loadFinished(!m_pageLoaded);
      return;
    }

  stop();

  QWidget *prevWidget = currentWidget();

#ifndef DOOBLE_X11_FLASH_BYPASS
  if(qobject_cast<dwebview *> (widget))
#else
  if(qobject_cast<QGraphicsView *> (widget))
#endif
    {
#ifndef DOOBLE_X11_FLASH_BYPASS
      setCurrentWidget(webView);
#else
      setCurrentWidget(m_view);
#endif

      bool doLoad = false;

      if(item.url().toString(QUrl::StripTrailingSlash) !=
	 webView->history()->forwardItem().url().
	 toString(QUrl::StripTrailingSlash))
	/*
	** QWebView's history's first forward item may be the current url.
	*/

	doLoad = true;

      if(prevWidget == currentWidget())
	{
	  bool emitSignals = false;
	  bool url1HasFragment = false;
	  bool url2HasFragment = false;
	  QUrl url1(webView->url());
	  QUrl url2(webView->history()->forwardItem().url());

	  /*
	  ** If we're going from an URL that has a fragment
	  ** to an URL that is equal to the current URL without
	  ** the fragment, we need to emit signals. Similarly
	  ** for the reverse.
	  */

	  if((url1HasFragment = url1.hasFragment()))
	    url1.setFragment(QString());

	  if((url2HasFragment = url2.hasFragment()))
	    url2.setFragment(QString());

	  if((url1 == url2 && (url1HasFragment || url2HasFragment)) ||
	     url2.isEmpty())
	    emitSignals = true;

	  webView->forward();

	  if(emitSignals)
	    {
	      m_pageLoaded = true;
	      emit loadFinished(m_pageLoaded);
	    }

	  if(url2HasFragment)
	    doLoad = true;
	}
      else if(!doLoad)
	load(item.url());

      if(doLoad)
	load(item.url());
    }
  else if(qobject_cast<dftpbrowser *> (widget))
    {
      setCurrentWidget(ftpBrowser);

      if(prevWidget == currentWidget())
	ftpBrowser->forward();
      else
	load(item.url());
    }
  else if(qobject_cast<dfilemanager *> (widget))
    {
      setCurrentWidget(fileManager);

      if(prevWidget == currentWidget())
	fileManager->forward();
      else
	load(item.url());
    }
}

QString dview::title(void) const
{
#ifndef DOOBLE_X11_FLASH_BYPASS
  if(qobject_cast<dwebview *> (currentWidget()))
#else
  if(qobject_cast<QGraphicsView *> (currentWidget()))
#endif
    return webView->title().trimmed();
  else if(qobject_cast<dftpbrowser *> (currentWidget()))
    return ftpBrowser->title().trimmed();
  else if(qobject_cast<dfilemanager *> (currentWidget()))
    return fileManager->title().trimmed();

  return QString();
}

void dview::slotUrlChanged(const QUrl &url)
{
  if(url.isEmpty() || !url.isValid())
    return;

  /*
  ** What about anchors?
  */

  bool fragment = false;
  QUrl url1(url);
  QUrl url2(m_history->currentItem().url());

  if(url1.hasFragment())
    {
      fragment = true;
      url1.setFragment(QString());
    }

  if(url2.hasFragment())
    {
      fragment = true;
      url2.setFragment(QString());
    }

  findIpAddress();
  emit urlChanged(url);

  if(url.scheme().toLower() == "http" ||
     url.scheme().toLower() == "https")
    {
      if(dooble::s_javaScriptExceptionsWindow->allowed(url.host()))
	/*
	** Exempt hosts must be prevented from executing JavaScript.
	*/

	webView->settings()->setAttribute(QWebSettings::JavascriptEnabled,
					  false);
      else
	webView->settings()->setAttribute
	  (QWebSettings::JavascriptEnabled,
	   dooble::s_settings.value("settingsWindow/javascriptEnabled", true).
	   toBool());

      webView->setProperty("temporary_url", url);
    }

  if(fragment &&
     url1.toString(QUrl::StripTrailingSlash) ==
     url2.toString(QUrl::StripTrailingSlash))
    {
      /*
      ** Required for populating navigation menus.
      */

      m_pageLoaded = true;
      emit loadFinished(m_pageLoaded);
    }
}

void dview::slotTitleChanged(const QString &title)
{
  emit titleChanged(title);
}

void dview::slotIconChanged(void)
{
  emit iconChanged();
}

bool dview::canGoBack(void) const
{
  return m_history->canGoBack();
}

bool dview::canGoForward(void) const
{
  return m_history->canGoForward();
}

QList<dwebhistoryitem> dview::backItems(const int n) const
{
  return m_history->backItems(n);
}

QList<dwebhistoryitem> dview::forwardItems(const int n) const
{
  return m_history->forwardItems(n);
}

void dview::print(QPrinter *printer)
{
  if(!printer)
    return;

  QWebFrame *frame = currentFrame();

  if(frame)
    frame->print(printer);
}

QWebFrame *dview::currentFrame(void)
{
#ifndef DOOBLE_X11_FLASH_BYPASS
  if(qobject_cast<dwebview *> (currentWidget()))
#else
  if(qobject_cast<QGraphicsView *> (currentWidget()))
#endif
    return webView->page()->currentFrame();
  else if(qobject_cast<dftpbrowser *> (currentWidget()))
    {
      initializeDirectoryView();
      directoryView->setHtml(ftpBrowser->html());
      return directoryView->page()->currentFrame();
    }
  else if(qobject_cast<dfilemanager *> (currentWidget()))
    {
      initializeDirectoryView();
      directoryView->setHtml(fileManager->html());
      return directoryView->page()->currentFrame();
    }

  return 0;
}

QString dview::html(void)
{
#ifndef DOOBLE_X11_FLASH_BYPASS
  if(qobject_cast<dwebview *> (currentWidget()))
#else
  if(qobject_cast<QGraphicsView *> (currentWidget()))
#endif
    return webView->page()->currentFrame()->toHtml();
  else if(qobject_cast<dftpbrowser *> (currentWidget()))
    {
      initializeDirectoryView();
      directoryView->setHtml(ftpBrowser->html());
      return directoryView->page()->currentFrame()->toHtml();
    }
  else if(qobject_cast<dfilemanager *> (currentWidget()))
    {
      initializeDirectoryView();
      directoryView->setHtml(fileManager->html());
      return directoryView->page()->currentFrame()->toHtml();
    }

  return "";
}

bool dview::isDir(void) const
{
  if(QFileInfo(url().toLocalFile()).isDir())
    return true;
  else
    return false;
}

bool dview::isFtp(void) const
{
  if(url().scheme().toLower() == "ftp")
    return true;
  else
    return false;
}

void dview::slotSslErrors(QNetworkReply *reply,
			  const QList<QSslError> &errors)
{
  /*
  ** Dooble must, at some point, allow the user to configure SSL
  ** certificates.
  */

  if(reply)
    reply->ignoreSslErrors(errors);
}

void dview::setFocus(void)
{
#ifndef DOOBLE_X11_FLASH_BYPASS
  if(qobject_cast<dwebview *> (currentWidget()))
    webView->setFocus();
#else
  if(qobject_cast<QGraphicsView *> (currentWidget()))
    {
      m_view->setFocus();
      webView->setFocus();
    }
#endif
  else if(qobject_cast<dftpbrowser *> (currentWidget()))
    ftpBrowser->setFocus();
  else if(qobject_cast<dfilemanager *> (currentWidget()))
    fileManager->setFocus();
}

void dview::slotHandleUnsupportedContent(const QUrl &url)
{
  emit saveUrl(url);
}

void dview::slotHandleUnsupportedContent(QNetworkReply *reply)
{
  if(reply && reply->error() == QNetworkReply::NoError)
    {
      if(reply->rawHeader("Content-Disposition").contains("inline") ||
	 reply->rawHeader("Content-Disposition").contains("attachment"))
	{
	  QString fileName(reply->rawHeader("Content-Disposition"));

	  if(fileName.indexOf('"') != -1 &&
	     fileName.lastIndexOf('"') != -1 &&
	     fileName.indexOf('"') != fileName.lastIndexOf('"'))
	    {
	      fileName = fileName.mid(fileName.indexOf('"') + 1);
	      fileName = fileName.mid(0, fileName.indexOf('"'));
	    }
	  else if(fileName.indexOf('<') != -1 &&
		  fileName.lastIndexOf('>') != -1 &&
		  fileName.indexOf('<') != fileName.lastIndexOf('>'))
	    {
	      fileName = fileName.mid(fileName.indexOf('<') + 1);
	      fileName = fileName.mid(0, fileName.indexOf('>'));
	    }
	  else
	    fileName = "dooble.download";

	  reply->abort();
	  emit saveFile(fileName, reply->url());
	}
      else
	{
	  if(reply->rawHeader("Content-Type").trimmed().toLower() ==
	     "application/octet-stream" &&
	     reply->rawHeader("Content-Length").toLongLong() > 0)
	    {
	      reply->abort();
	      emit saveUrl(reply->url());
	    }
	  else if(reply->rawHeader("Content-Type").trimmed().contains("text"))
	    {
	      reply->abort();
	      emit saveUrl(reply->url());
	    }
	  else if(reply->rawHeader("Content-Type").trimmed().contains
		  ("application"))
	    {
	      reply->abort();
	      emit saveUrl(reply->url());
	    }
	  else
	    reply->abort();
	}
    }
  else if(reply)
    reply->abort();
}

void dview::enterEvent(QEvent *event)
{
  QStackedWidget::enterEvent(event);
  emit viewEntered();
}

void dview::slotSelectionChanged(void)
{
  emit selectionChanged(webView->page()->selectedText());
}

dwebhistoryitem dview::currentItem(void) const
{
  dwebhistoryitem item(m_history->currentItem());

  /*
  ** In the unlikely event that the current item
  ** is not included in the history, create an item
  ** based on current data.
  */

  if(item.url().isEmpty())
    {
      item.setUrl(url());
      item.setIcon(icon());
      item.setTitle(title());
    }

  return item;
}

QWidget *dview::widgetForUrl(const QUrl &url)
{
  /*
  ** Determine the stack widget's correct
  ** widget based on the provided url.
  */

  if(url.isEmpty() || !url.isValid())
    return 0;

  if(url.scheme().toLower() == "http" ||
     url.scheme().toLower() == "https" ||
     url.scheme().toLower() == "qrc" ||
     ((url.scheme().toLower() == "file") &&
      QFileInfo(url.toLocalFile()).isFile()))
#ifndef DOOBLE_X11_FLASH_BYPASS
    return webView;
#else
    return m_view;
#endif
  else if(url.scheme().toLower() == "ftp")
    {
      initializeFtpBrowser();
      return ftpBrowser;
    }
  else if(url.scheme().toLower() == "file")
    {
      initializeFileManager();
      return fileManager;
    }
  else
    return 0;
}

void dview::slotPaste(void)
{
  webView->triggerPageAction(QWebPage::Paste);
}

void dview::setTabAction(QAction *action)
{
  if(m_action)
    {
      removeAction(m_action);
      m_action->deleteLater();
    }

  m_action = action;

  if(m_action)
    addAction(m_action);
}

QAction *dview::tabAction(void) const
{
  return m_action;
}

void dview::slotLinkClicked(const QUrl &url)
{
  load(url);
}

dwebview *dview::view(void) const
{
  return webView;
}

void dview::slotStop(void)
{
  stop();
}

void dview::slotViewPageSource(void)
{
  emit viewPageSource();
}

void dview::initializeFtpBrowser(void)
{
  if(!ftpBrowser)
    {
      ftpBrowser = new dftpbrowser(this);
      connect(ftpBrowser, SIGNAL(loadFinished(bool)), this,
	      SLOT(slotLoadFinished(bool)));
      connect(ftpBrowser, SIGNAL(loadStarted(void)), this,
	      SLOT(slotLoadStarted(void)));
      connect(ftpBrowser, SIGNAL(urlChanged(const QUrl &)),
	      this, SLOT(slotUrlChanged(const QUrl &)));
      connect(ftpBrowser, SIGNAL(titleChanged(const QString &)),
	      this, SLOT(slotTitleChanged(const QString &)));
      connect(ftpBrowser, SIGNAL(iconChanged(void)),
	      this, SLOT(slotIconChanged(void)));
      connect(ftpBrowser, SIGNAL(saveUrl(const QUrl &)),
	      this, SLOT(slotSaveLink(const QUrl &)));
      connect(ftpBrowser, SIGNAL(copyLink(const QUrl &)),
	      this, SLOT(slotCopyLinkLocation(const QUrl &)));
      connect(ftpBrowser, SIGNAL(openLinkInNewTab(const QUrl &)),
	      this, SLOT(slotOpenLinkInNewTab(const QUrl &)));
      connect(ftpBrowser, SIGNAL(openLinkInNewWindow(const QUrl &)),
	      this, SLOT(slotOpenLinkInNewWindow(const QUrl &)));
      connect(ftpBrowser, SIGNAL(loadProgress(int)), this,
	      SLOT(slotLoadProgress(int)));
      connect(ftpBrowser, SIGNAL(urlsReceivedViaDrop(const QList<QUrl> &)),
	      this, SIGNAL(urlsReceivedViaDrop(const QList<QUrl> &)));
      connect(ftpBrowser,
	      SIGNAL(unsupportedContent(const QUrl &)),
	      this,
	      SLOT(slotHandleUnsupportedContent(const QUrl &)));
      addWidget(ftpBrowser);
    }
}

void dview::initializeFileManager(void)
{
  if(!fileManager)
    {
      fileManager = new dfilemanager(this);
      connect(fileManager, SIGNAL(loadFinished(bool)), this,
	      SLOT(slotLoadFinished(bool)));
      connect(fileManager, SIGNAL(loadStarted(void)), this,
	      SLOT(slotLoadStarted(void)));
      connect(fileManager, SIGNAL(urlChanged(const QUrl &)),
	      this, SLOT(slotUrlChanged(const QUrl &)));
      connect(fileManager, SIGNAL(titleChanged(const QString &)),
	      this, SLOT(slotTitleChanged(const QString &)));
      connect(fileManager, SIGNAL(iconChanged(void)),
	      this, SLOT(slotIconChanged(void)));
      connect(fileManager, SIGNAL(loadProgress(int)),
	      this, SLOT(slotLoadProgress(int)));
      connect(fileManager, SIGNAL(urlsReceivedViaDrop(const QList<QUrl> &)),
	      this, SIGNAL(urlsReceivedViaDrop(const QList<QUrl> &)));
      addWidget(fileManager);
    }
}

void dview::initializeDirectoryView(void)
{
  if(!directoryView)
    {
      directoryView = new QWebView(this);
      directoryView->setVisible(false);
    }
}

void dview::slotSetTextSizeMultiplier(const qreal multiplier)
{
#ifndef DOOBLE_X11_FLASH_BYPASS
  webView->setTextSizeMultiplier(multiplier);
#else
  foreach(QWebFrame *frame, webView->page()->findChildren<QWebFrame *> ())
    frame->setTextSizeMultiplier(multiplier);
#endif
}

bool dview::hasSecureSonnection(void) const
{
  /*
  ** Simple, for now.
  */

  return (url().scheme().toLower() == "https");
}

void dview::findIpAddress(void)
{
  QHostInfo::abortHostLookup(m_lastInfoLookupId);

  if(dooble::s_settings.value("settingsWindow/displayIpAddress", false).
     toBool())
    {
      if(url().isValid())
	{
	  if(url().host().isEmpty())
	    m_lastInfoLookupId = QHostInfo::lookupHost
	      ("localhost", this,
	       SLOT(slotHostLookedUp(const QHostInfo &)));
	  else
	    m_lastInfoLookupId = QHostInfo::lookupHost
	      (url().host(), this,
	       SLOT(slotHostLookedUp(const QHostInfo &)));
	}
      else
	{
	  m_ipAddress = "";
	  emit ipAddressChanged("");
	}
    }
  else
    {
      m_ipAddress = "";
      emit ipAddressChanged("");
    }
}

void dview::slotHostLookedUp(const QHostInfo &hostInfo)
{
  if(dooble::s_settings.value("settingsWindow/displayIpAddress", false).
     toBool())
    foreach(const QHostAddress &address, hostInfo.addresses())
      if(!address.isNull())
	{
	  m_ipAddress = address.toString();
	  emit ipAddressChanged(m_ipAddress);
	  break;
	}
}

QString dview::ipAddress(void) const
{
  return m_ipAddress;
}

bool dview::isModified(void) const
{
#ifndef DOOBLE_X11_FLASH_BYPASS
  if(qobject_cast<dwebview *> (currentWidget()))
#else
  if(qobject_cast<QGraphicsView *> (currentWidget()))
#endif
    return webView->isModified();
  else
    return false;
}

void dview::post(const QUrl &url, const QString &text)
{
  QNetworkRequest request;

  if(dooble::s_javaScriptExceptionsWindow->allowed(url.host()))
    /*
    ** Exempt hosts must be prevented from executing JavaScript.
    */

    webView->settings()->setAttribute(QWebSettings::JavascriptEnabled,
				      false);
  else
    webView->settings()->setAttribute
      (QWebSettings::JavascriptEnabled,
       dooble::s_settings.value("settingsWindow/javascriptEnabled", true).
       toBool());

  /*
  ** We don't need to set HTTP Pipelining here since
  ** dnetworkaccessmanager::createRequest() will set it.
  */

  request.setUrl(url);
  webView->load(request,
		QNetworkAccessManager::PostOperation,
		text.toUtf8());
}

#ifdef DOOBLE_X11_FLASH_BYPASS
void dview::resizeEvent(QResizeEvent *event)
{
  if(event)
    {
      webView->resize(event->size());
      m_view->setSceneRect(0, 0,
			   webView->size().width(),
			   webView->size().height());
    }

  QStackedWidget::resizeEvent(event);
}
#endif

void dview::update(void)
{
#ifdef DOOBLE_X11_FLASH_BYPASS
  if(isVisible())
    m_view->fitInView(webView);
#endif

  QStackedWidget::update();
}

dwebhistory *dview::history(void) const
{
  return m_history;
}
