/****************************************************************************
** Dooble - The Secure Internet Web Browser
**
** Copyright (c) 2008, 2009, 2010, 2011 Alexis Megas,
** Gunther van Dooble, and the Dooble Team.
** All rights reserved.
**
** License: GPL2 only:
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; version 2 of the License only.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
** or see here: http://www.gnu.org/licenses/gpl.html
**
** For the WebKit library, please see: http://webkit.org.
**
** THE CODE IS PROVIDED BY THE AUTHORS ''AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
** ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
** GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
** IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS APPLICATION, EVEN IF ADVISED
** OF THE POSSIBILITY OF SUCH DAMAGE.
**
** Please report all praise, requests, bugs, and problems to the project
** team and administrators: http://sf.net/projects/dooble.
**
** You can find us listed at our project page. New team members are welcome.
** The name of the authors should not be used to endorse or promote products
** derived from Dooble without specific prior written permission.
** If you use this code for other projects, please let us know.
**
** Web sites:
**   http://sf.net/projects/dooble
**   http://dooble.sf.net
****************************************************************************/

#include <QtDebug>
#include <QDropEvent>
#include <QMouseEvent>
#include <QNetworkReply>
#include <QWebHitTestResult>
#include <QWebElementCollection>
#ifdef DOOBLE_X11_FLASH_BYPASS
#include <QGraphicsSceneDragDropEvent>
#endif

#include "dwebview.h"

#ifndef DOOBLE_X11_FLASH_BYPASS
dwebview::dwebview(QWidget *parent):QWebView(parent)
#else
dwebview::dwebview(QGraphicsItem *parent):QGraphicsWebView(parent)
#endif
{
  m_allowPopup = false;
  m_faviconManager = new QNetworkAccessManager(this);
#ifdef DOOBLE_X11_FLASH_BYPASS
  setFocusPolicy(Qt::StrongFocus);
  setWindowFlags(Qt::FramelessWindowHint);
#endif
  connect(this,
	  SIGNAL(urlChanged(const QUrl &)),
	  this,
	  SLOT(slotUrlChanged(const QUrl &)));
  connect(this,
	  SIGNAL(loadStarted(void)),
	  this,
	  SLOT(slotLoadStarted(void)));
}

bool dwebview::checkAndClearPopup(void)
{
  bool allowPopup = m_allowPopup;

  m_allowPopup = false;
  return allowPopup;
}

#ifndef DOOBLE_X11_FLASH_BYPASS
void dwebview::dropEvent(QDropEvent *event)
#else
void dwebview::dropEvent(QGraphicsSceneDragDropEvent *event)
#endif
{
  if(event)
    {
      if(event->mimeData() && event->mimeData()->hasUrls())
	{
	  QList<QUrl> list(event->mimeData()->urls());
	  emit urlsReceivedViaDrop(list);
	  list.clear();
	  return;
	}
    }

#ifndef DOOBLE_X11_FLASH_BYPASS
  QWebView::dropEvent(event);
#else
  QGraphicsWebView::dropEvent(event);
#endif
}

#ifndef DOOBLE_X11_FLASH_BYPASS
void dwebview::mousePressEvent(QMouseEvent *event)
#else
void dwebview::mousePressEvent(QGraphicsSceneMouseEvent *event)
#endif
{
  /*
  ** We use this method to determine valid popups.
  */

  m_allowPopup = false;

  if(event)
    {
      QWebFrame *frame = page()->currentFrame();

      if(frame)
	{
	  QWebHitTestResult hit = frame->hitTestContent
#ifndef DOOBLE_X11_FLASH_BYPASS
	    (event->pos());
#else
	    (event->pos().toPoint());
#endif

	  if(hit.linkUrl().isValid())
	    m_allowPopup = true;
	  else if(!hit.isNull())
	    {
	      bool found = false;
	      QWebElementCollection collection = frame->
		documentElement().findAll("input[type=button]");

	      foreach(QWebElement element, collection)
		{
		  if(element == hit.element())
		    {
		      found = true;
		      break;
		    }
		}

	      if(!found)
		{
		  collection = frame->
		    documentElement().findAll("input[type=image]");

		  foreach(QWebElement element, collection)
		    {
		      if(element == hit.element())
			{
			  found = true;
			  break;
			}
		    }
		}

	      if(!found)
		{
		  collection = frame->
		    documentElement().findAll("input[type=submit]");

		  foreach(QWebElement element, collection)
		    {
		      if(element == hit.element())
			{
			  found = true;
			  break;
			}
		    }
		}

	      if(found)
		m_allowPopup = true;
	      else
		{
		  foreach(QWebElement element, frame->documentElement().
			  findAll("*"))
		    if(element.geometry().contains(hit.pos()))
		      {
			if(element.attribute("role").toLower() == "link" ||
			   element.attribute("method").toLower() == "post" ||
			   element.hasAttribute("href") ||
			   element.hasAttribute("onclick") ||
			   (element.hasAttribute("id") &&
			    QUrl(element.attribute("id")).isValid()) ||
			   (element.hasAttribute("src") &&
			    QUrl(element.attribute("src")).isValid()))
			  {
			    found = true;
			    break;
			  }
		      }

		  if(found)
		    m_allowPopup = true;
		}
	    }
	}
    }

#ifndef DOOBLE_X11_FLASH_BYPASS
  QWebView::mousePressEvent(event);
#else
  QGraphicsWebView::mousePressEvent(event);
#endif
}

QIcon dwebview::icon(void) const
{
  return m_icon;
}

void dwebview::slotUrlChanged(const QUrl &url)
{
  downloadFavicon(url);
}

void dwebview::slotLoadStarted(void)
{
}

void dwebview::downloadFavicon(const QUrl &url)
{
  if(url.scheme().toLower() == "http" ||
     url.scheme().toLower() == "https")
    {
      foreach(QNetworkReply *reply,
	      m_faviconManager->findChildren<QNetworkReply *> ())
	reply->abort();

      QUrl faviconUrl;

      faviconUrl.setHost(url.host());
      faviconUrl.setPath("favicon.ico");
      faviconUrl.setScheme(url.scheme().toLower());

      QNetworkReply *reply = 0;
      QNetworkRequest request(faviconUrl);

      request.setAttribute(QNetworkRequest::HttpPipeliningAllowedAttribute,
			   true);
      reply = m_faviconManager->get(request);
      reply->ignoreSslErrors();
      connect(reply, SIGNAL(finished(void)),
	      this, SLOT(slotIconDownloadFinished(void)));
    }
}

void dwebview::slotIconDownloadFinished(void)
{
  QNetworkReply *reply = qobject_cast<QNetworkReply *> (sender());

  if(reply)
    {
      QUrl url(reply->attribute(QNetworkRequest::RedirectionTargetAttribute).
	       toUrl());

      /*
      ** The URL may be relative.
      */

      url = reply->url().resolved(url);

      if(!url.isEmpty() && url.isValid() &&
	 url.toString(QUrl::StripTrailingSlash) !=
	 reply->url().toString(QUrl::StripTrailingSlash))
	{
	  url.setPath("favicon.ico");

	  QNetworkReply *reply = 0;
	  QNetworkRequest request(url);

	  request.setAttribute
	    (QNetworkRequest::HttpPipeliningAllowedAttribute, true);
	  reply = m_faviconManager->get(request);
	  reply->ignoreSslErrors();
	  connect(reply, SIGNAL(finished(void)),
		  this, SLOT(slotIconDownloadFinished(void)));
	}
      else
	{
	  QPixmap pixmap;
	  QByteArray bytes(reply->readAll());

	  if(pixmap.loadFromData(bytes, "gif"))
	    m_icon = QIcon(pixmap);
	  else if(pixmap.loadFromData(bytes, "ico"))
	    m_icon = QIcon(pixmap);
	  else if(pixmap.loadFromData(bytes, "jpg"))
	    m_icon = QIcon(pixmap);
	  else if(pixmap.loadFromData(bytes, "jpeg"))
	    m_icon = QIcon(pixmap);
	  else if(pixmap.loadFromData(bytes, "png"))
	    m_icon = QIcon(pixmap);
	  else
	    m_icon = QIcon();

	  emit iconChanged();
	}

      reply->deleteLater();
    }
}
